''' <h1>Reflectivity plugin </h1>
Reflectivity is a plugin for providing a graphical user
interface to define multilayer structures in GenX. It works
on quite general principels with dynamic generation of the
graphical user interface depending on the model. It also
dynamically generates the script for the model. Thus, it is 
always possible to go in and edit the script manually. <p>

The plugin consists of the following components:
<h2>Sample tab</h2>
This tab has the definitons for the layers and stacks. 
remember that a layer has to be inside a stack. 
Also, the name of the layers must be uniqe and can not be change
after the layer has been created. The functions of the buttons 
from left to right are:
<dl>
    <dt><b>Add Layer</b></dt>
    <dd>Add a new layer to the current position</dd>
    <dt><b>Add Stack</b></dt>
    <dd>Add a new stack to the current position</dd>
    <dt><b>Remove item</b></dt>
    <dd>Removes the current item. Note that the substrate, Sub, and the 
    ambient material can not be removed.</dd>
    <dt><b>Move item up</b></dt>
    <dd>Move item up</dd>
    <dt><b>Move item down</b></dt>
    <dd>Move item down</dd>
    <dt><b>Sample parameters</b></dt>
    <dd>Edit global parameters for the entire sample</dd>
    <dt><b>Instrument</b></dt>
    <dd>Edit parameters such as resolution and incident intesnity that
    is defines the instrument.</dd>
</dl>
<h2>Simulation tab</h2>
Here it is possible to add commands that are conducted before a data 
set is calculated. This done by adding a new command by pressing the green
plus sign. This brings up a dialog where the object and paraemter can
be chosen and the expression typed in. Note that the list one can choose from 
is <b>only</b> updated when the simulation button is pressed.<p>

The blue nut button to the right brings up a menu that allows the definition
of custom variables. These can be used to define problem specific parameters
 such as, for example, compostion of layers. One can also use this for parameter
coupling that yields a speedup in fitting. For example, fitting the repetition
length for a multilayer. 

<h2>SLD tab</h2>
This shows the real and imaginary part of the scattering length as a function
of depth for the sample. The substrate is to the left and the ambient material
is to the right. This is updated when the simulation button is pressed.
'''

from .. import add_on_framework as framework
from genx.plotpanel import PlotPanel
import genx.model as modellib
import wx.html

import numpy as np
import sys, os, re, time, io, traceback

from .help_modules.custom_dialog import *
from .help_modules import reflectivity_images as images
from .help_modules.reflectivity_utils import SampleHandler, SampleBuilder, avail_models, find_code_segment
from genx.gui_logging import iprint

_set_func_prefix='set'

class MyHtmlListBox(wx.html.HtmlListBox):

    def __init__(self, parent, id, size=(-1, -1), style=wx.BORDER_SUNKEN):
        wx.html.HtmlListBox.__init__(self, parent, id, size=size,
                                     style=style)
        self.SetItemList(['Starting up...'])

    def SetItemList(self, list):
        self.html_items=list
        self.SetItemCount(len(list))
        # self.RefreshAll()
        self.Refresh()

    def OnGetItem(self, n):
        return self.html_items[n]

class SamplePanel(wx.Panel):
    def __init__(self, parent, plugin, refindexlist=None):
        wx.Panel.__init__(self, parent)
        if refindexlist is None:
            refindexlist=[]
        self.refindexlist=refindexlist
        self.plugin=plugin
        self.variable_span=0.25

        # Colours indicating different states
        # Green wx.Colour(138, 226, 52), ORANGE wx.Colour(245, 121, 0)
        self.fit_colour=(245, 121, 0)
        # Tango Sky blue wx.Colour(52, 101, 164), wx.Colour(114, 159, 207)
        self.const_fit_colour=(114, 159, 207)

        boxver=wx.BoxSizer(wx.HORIZONTAL)
        boxhor=wx.BoxSizer(wx.VERTICAL)
        self.toolbar=wx.ToolBar(self, style=wx.TB_FLAT | wx.TB_HORIZONTAL)
        boxhor.Add((-1, 2))
        self.do_toolbar()
        boxhor.Add(self.toolbar, proportion=0, flag=wx.EXPAND, border=1)
        boxhor.Add((-1, 2))
        self.listbox=MyHtmlListBox(self, -1, style=wx.BORDER_SUNKEN)
        # self.listbox.SetItemList(self.sampleh.getStringList())
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.lbDoubleClick, self.listbox)
        boxhor.Add(self.listbox, 1, wx.EXPAND)

        boxver.Add(boxhor, 1, wx.EXPAND)

        self.SetSizer(boxver)
        self.toolbar.Realize()
        self.update_callback=lambda event: ''

    def do_toolbar(self):
        dpi_scale_factor=wx.GetApp().dpi_scale_factor
        tb_bmp_size=int(dpi_scale_factor*20)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Insert Layer',
                             bitmap=wx.Bitmap(images.insert_layer.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Insert a Layer')
        self.Bind(wx.EVT_TOOL, self.InsertLay, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Insert Stack',
                             bitmap=wx.Bitmap(images.insert_stack.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Insert a Stack')
        self.Bind(wx.EVT_TOOL, self.InsertStack, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Delete',
                             bitmap=wx.Bitmap(images.delete.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Delete item')
        self.Bind(wx.EVT_TOOL, self.DeleteSample, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Rename',
                             bitmap=wx.Bitmap(images.change_name.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Rename')
        self.Bind(wx.EVT_TOOL, self.ChangeName, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Move up',
                             bitmap=wx.Bitmap(images.move_up.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Move item up')
        self.Bind(wx.EVT_TOOL, self.MoveUp, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Move down',
                             bitmap=wx.Bitmap(images.move_down.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Move item down')
        self.Bind(wx.EVT_TOOL, self.MoveDown, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Edit Sample',
                             bitmap=wx.Bitmap(images.sample.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Edit Sample parameters')
        self.Bind(wx.EVT_TOOL, self.EditSampleParameters, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, 'Edit Instrument',
                             bitmap=wx.Bitmap(images.instrument.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Edit Instruments')
        self.Bind(wx.EVT_TOOL, self.EditInstrument, id=newid)

    def SetUpdateCallback(self, func):
        ''' SetUpdateCallback(self, func) --> None
        
        Sets the update callback will be called when the sample is updated.
        The call is on the form func(event)
        '''
        self.update_callback=func

    def create_html_decorator(self):
        """
        creates a html decorator function
        :return:
        """
        grid_parameters=self.plugin.GetModel().get_parameters()
        dic_lookup={}
        for par in grid_parameters.get_names():
            l=par.split('.')
            if len(l)==2:
                name=l[0]
                par_name=l[1][3:].lower()
                dic_lookup[(name, par_name)]=(grid_parameters.get_value_by_name(par),
                                              grid_parameters.get_fit_state_by_name(par)
                                              )
        fit_color_str="rgb(%d,%d,%d)"%self.fit_colour
        const_fit_color_str="rgb(%d,%d,%d)"%self.const_fit_colour

        def decorator(name, str):
            """ Decorator to indicate the parameters that are fitted"""
            try:
                start_index=str.index('(')+1
            except ValueError:
                start_index=0
            ret_str=str[:start_index]
            for par_str in str[start_index:].split(','):
                par_name=par_str.split('=')[0].strip()
                # par_name normal paramter (real number)
                if (name, par_name) in dic_lookup:
                    val, state=dic_lookup[(name, par_name)]
                    if state==1:
                        par_str=' <font color=%s><b>%s=%.2e</b></font>,'%(fit_color_str, par_name, val)
                    elif state==2:
                        par_str=' <font color=%s><b>%s=%.2e</b></font>,'%(const_fit_color_str, par_name, val)
                # par_name is a complex parameter...
                elif (name, par_name+'real') in dic_lookup or (name, par_name+'imag') in dic_lookup:
                    if (name, par_name+'real') in dic_lookup:
                        val, state=dic_lookup[(name, par_name+'real')]
                        if state==1:
                            par_str=' <font color=%s><b>%s=(%.2e,</b></font>'%(fit_color_str, par_name, val)
                        elif state==2:
                            par_str=' <font color=%s><b>%s=(%.2e,</b></font>'%(const_fit_color_str, par_name, val)
                    else:
                        par_str=' <b>%s=??+</b>'%par_name
                    if (name, par_name+'imag') in dic_lookup:
                        val, state=dic_lookup[(name, par_name+'imag')]
                        if state==1:
                            par_str+=' <font color=%s><b>%.2e)</b></font>,'%(fit_color_str, val)
                        elif state==2:
                            par_str+=' <font color=%s><b>%.2e)</b></font>,'%(const_fit_color_str, val)
                    else:
                        par_str+=' <b>??)</b>,'

                else:
                    par_str+=','
                ret_str+=par_str
            # Remove trailing ,
            if ret_str[-1]==',':
                ret_str=ret_str[:-1]
            if str[-1]==')' and ret_str[-1]!=')':
                ret_str+=')'
            return ret_str

        return decorator

    def Update(self, update_script=True):
        deco=self.create_html_decorator()
        sl=self.sampleh.getStringList(html_encoding=True, html_decorator=deco)
        self.listbox.SetItemList(sl)
        if update_script:
            self.update_callback(None)

    def SetSample(self, sample, names):
        self.sampleh.sample=sample
        self.sampleh.names=names
        self.Update()

    def EditSampleParameters(self, evt):
        """ Event handler that creates a dialog box to edit the sample parameters.

        :param evt:
        :return: Nothing
        """
        obj_name='sample'
        eval_func=self.plugin.GetModel().eval_in_model
        grid_parameters=self.plugin.GetModel().get_parameters()

        validators={}
        vals={}
        pars=[]
        items=[]
        editable={}
        try:
            string_choices=self.model.sample_string_choices
        except Exception as e:
            string_choices={}
        for item in self.model.SampleParameters:
            if item!='Stacks' and item!='Substrate' and item!='Ambient':
                if item in string_choices:
                    validators[item]=string_choices[item]
                else:
                    validators[item]=FloatObjectValidator()
                val=getattr(self.sampleh.sample, item)
                vals[item]=val
                pars.append(item)
                items.append((item, val))
                # Check if the parameter is in the grid and in that case set it as uneditable
                func_name=obj_name+'.'+_set_func_prefix+item.capitalize()
                grid_value=grid_parameters.get_value_by_name(func_name)
                editable[item]=grid_parameters.get_fit_state_by_name(func_name)
                if grid_value is not None:
                    vals[item]=grid_value
        try:
            groups=self.model.SampleGroups
        except Exception:
            groups=False
        try:
            units=self.model.SampleUnits
        except Exception:
            units=False

        dlg=ValidateFitDialog(self, pars, vals, validators,
                              title='Sample Editor', groups=groups,
                              units=units, editable_pars=editable)

        if dlg.ShowModal()==wx.ID_OK:
            old_vals=vals
            vals=dlg.GetValues()
            # print vals
            states=dlg.GetStates()
            for par in pars:
                if not states[par]:
                    old_type=type(old_vals[par])
                    setattr(self.sampleh.sample, par, old_type(vals[par]))
                if editable[par]!=states[par]:
                    value=eval_func(vals[par])
                    minval=min(value*(1-self.variable_span), value*(1+self.variable_span))
                    maxval=max(value*(1-self.variable_span), value*(1+self.variable_span))
                    func_name=obj_name+'.'+_set_func_prefix+par.capitalize()
                    grid_parameters.set_fit_state_by_name(func_name, value, states[par], minval, maxval)
                    # Tell the grid to reload the parameters
                    self.plugin.parent.paramter_grid.SetParameters(grid_parameters)

            self.Update()

        dlg.Destroy()

    def SetInstrument(self, instruments):
        '''SetInstrument(self, instrument) --> None
        
        Sets the instruments should be a dictionary of instruments with the key being the
        name of the instrument
        '''
        self.instruments=instruments

    def EditInstrument(self, evt):
        """Event handler that creates an dialog box to edit the instruments.

        :param evt:
        :return: Nothing
        """
        eval_func=self.plugin.GetModel().eval_in_model
        validators={}
        vals={}
        editable={}
        grid_parameters=self.plugin.GetModel().get_parameters()
        for inst_name in self.instruments:
            vals[inst_name]={}
            editable[inst_name]={}

        pars=[]
        for item in self.model.InstrumentParameters:
            if item in self.model.instrument_string_choices:
                # validators.append(self.model.instrument_string_choices[item])
                validators[item]=self.model.instrument_string_choices[item]
            else:
                # validators.append(FloatObjectValidator())
                validators[item]=FloatObjectValidator()
            for inst_name in self.instruments:
                val=getattr(self.instruments[inst_name], item)
                vals[inst_name][item]=val
                # Check if the parameter is in the grid and in that case set it as uneditable
                func_name=inst_name+'.'+_set_func_prefix+item.capitalize()
                grid_value=grid_parameters.get_value_by_name(func_name)
                editable[inst_name][item]=grid_parameters.get_fit_state_by_name(func_name)
                if grid_value is not None:
                    vals[inst_name][item]=grid_value
            pars.append(item)

        old_insts=[]
        for inst_name in self.instruments:
            old_insts.append(inst_name)

        try:
            groups=self.model.InstrumentGroups
        except Exception:
            groups=False
        try:
            units=self.model.InstrumentUnits
        except Exception:
            units=False
        dlg=ValidateFitNotebookDialog(self, pars, vals, validators,
                                      title='Instrument Editor', groups=groups,
                                      units=units, fixed_pages=['inst'], editable_pars=editable)

        if dlg.ShowModal()==wx.ID_OK:
            old_vals=vals
            vals=dlg.GetValues()
            # print vals
            states=dlg.GetStates()
            self.instruments={}
            for inst_name in vals:
                new_instrument=False
                if inst_name not in self.instruments:
                    # A new instrument must be created:
                    self.instruments[inst_name]=self.model.Instrument()
                    new_instrument=True
                for par in self.model.InstrumentParameters:
                    if not states[inst_name][par]:
                        old_type=type(old_vals[inst_name][par])
                        setattr(self.instruments[inst_name], par, old_type(vals[inst_name][par]))
                    else:
                        setattr(self.instruments[inst_name], par, old_vals[inst_name][par])
                    if new_instrument and states[inst_name][par]>0:
                        value=eval_func(vals[inst_name][par])
                        minval=min(value*(1-self.variable_span), value*(1+self.variable_span))
                        maxval=max(value*(1-self.variable_span), value*(1+self.variable_span))
                        func_name=inst_name+'.'+_set_func_prefix+par.capitalize()
                        grid_parameters.set_fit_state_by_name(func_name, value, states[inst_name][par], minval, maxval)
                    elif not new_instrument:
                        if editable[inst_name][par]!=states[inst_name][par]:
                            value=eval_func(vals[inst_name][par])
                            minval=min(value*(1-self.variable_span), value*(1+self.variable_span))
                            maxval=max(value*(1-self.variable_span), value*(1+self.variable_span))
                            func_name=inst_name+'.'+_set_func_prefix+par.capitalize()
                            grid_parameters.set_fit_state_by_name(func_name, value, states[inst_name][par], minval,
                                                                  maxval)

            # Loop to remove instrument from grid if not returned from Dialog
            for inst_name in old_insts:
                if inst_name not in list(vals.keys()):
                    for par in self.model.InstrumentParameters:
                        if editable[inst_name][par]>0:
                            func_name=inst_name+'.'+_set_func_prefix+par.capitalize()
                            grid_parameters.set_fit_state_by_name(func_name, 0, 0, 0, 0)

            # Tell the grid to reload the parameters
            self.plugin.parent.paramter_grid.SetParameters(grid_parameters)

            for change in dlg.GetChanges():
                if change[0]!='' and change[1]!='':
                    self.plugin.InstrumentNameChange(change[0], change[1])
                elif change[1]=='':
                    self.plugin.InstrumentNameChange(change[0], 'inst')

            self.Update()
        else:
            pass
        dlg.Destroy()

    def MoveUp(self, evt):
        sl=self.sampleh.moveUp(self.listbox.GetSelection())
        if sl:
            self.Update()
            self.listbox.SetSelection(self.listbox.GetSelection()-1)

    def MoveDown(self, evt):
        sl=self.sampleh.moveDown(self.listbox.GetSelection())
        if sl:
            self.Update()
            self.listbox.SetSelection(self.listbox.GetSelection()+1)

    def InsertStack(self, evt):
        # Create Dialog box
        items=[('Name', 'name')]
        validators={}
        vals={}
        validators['Name']=NoMatchValidTextObjectValidator(self.sampleh.names)
        pars=['Name']
        vals['Name']='name'
        dlg=ValidateDialog(self, pars, vals, validators,
                           title='Give Stack Name')

        # Show the dialog
        if dlg.ShowModal()==wx.ID_OK:
            vals=dlg.GetValues()
        dlg.Destroy()
        # if not a value is selected operate on first
        pos=max(self.listbox.GetSelection(), 0)
        sl=self.sampleh.insertItem(pos, 'Stack', vals['Name'])
        if sl:
            self.Update()
        else:
            self.plugin.ShowWarningDialog('Can not insert a stack at the'
                                          ' current position.')

    def InsertLay(self, evt):
        # Create Dialog box
        # items = [('Name', 'name')]
        # validators = [NoMatchValidTextObjectValidator(self.sampleh.names)]
        dlg=ValidateDialog(self, ['Name'], {'Name': 'name'},
                           {'Name': NoMatchValidTextObjectValidator(self.sampleh.names)},
                           title='Give Layer Name')
        # Show the dialog
        if dlg.ShowModal()==wx.ID_OK:
            vals=dlg.GetValues()
        dlg.Destroy()
        # if not a value is selected operate on first
        pos=max(self.listbox.GetSelection(), 0)
        # Create the Layer
        sl=self.sampleh.insertItem(pos, 'Layer', vals['Name'])
        if sl:
            self.Update()
        else:
            self.plugin.ShowWarningDialog('Can not insert a layer at the'
                                          ' current position. Layers has to be part of a stack.')

    def DeleteSample(self, evt):
        slold=self.sampleh.getStringList()
        sl=self.sampleh.deleteItem(self.listbox.GetSelection())
        if sl:
            self.Update()

    def ChangeName(self, evt):
        '''Change the name of the current selected item.
        '''
        pos=self.listbox.GetSelection()
        if pos==0 or pos==len(self.sampleh.names)-1:
            self.plugin.ShowInfoDialog('It is forbidden to change the'
                                       'name of the substrate (Sub) and the Ambient (Amb) layers.')
        else:
            unallowed_names=self.sampleh.names[:pos]+ \
                            self.sampleh.names[max(0, pos-1):]
            dlg=ValidateDialog(self, ['Name'], {'Name': self.sampleh.names[pos]},
                               {'Name': NoMatchValidTextObjectValidator(unallowed_names)},
                               title='Give New Name')

            if dlg.ShowModal()==wx.ID_OK:
                vals=dlg.GetValues()
                result=self.sampleh.changeName(pos, vals['Name'])
                if result:
                    self.Update()
                else:
                    iprint('Unexpected problems when changing name...')
            dlg.Destroy()

    def lbDoubleClick(self, evt):
        sel=self.sampleh.getItem(self.listbox.GetSelection())
        obj_name=self.sampleh.getName(self.listbox.GetSelection())
        eval_func=self.plugin.GetModel().eval_in_model
        sl=None
        items=[]
        validators={}
        vals={}
        pars=[]
        editable={}
        grid_parameters=self.plugin.GetModel().get_parameters()
        if isinstance(sel, self.model.Layer):
            # The selected item is a Layer
            for item in list(self.model.LayerParameters.keys()):
                value=getattr(sel, item)
                vals[item]=value
                # if item!='n' and item!='fb':
                if type(self.model.LayerParameters[item])!=type(1+1.0J):
                    # Handle real parameters
                    validators[item]=FloatObjectValidator(eval_func, alt_types=[self.model.Layer])
                    func_name=obj_name+'.'+_set_func_prefix+item.capitalize()
                    grid_value=grid_parameters.get_value_by_name(func_name)
                    if grid_value is not None:
                        vals[item]=grid_value
                    editable[item]=grid_parameters.get_fit_state_by_name(func_name)

                else:
                    # Handle complex parameters
                    validators[item]=ComplexObjectValidator(eval_func, alt_types=[self.model.Layer])
                    func_name=obj_name+'.'+_set_func_prefix+item.capitalize()
                    grid_value_real=grid_parameters.get_value_by_name(func_name+'real')
                    grid_value_imag=grid_parameters.get_value_by_name(func_name+'imag')
                    if grid_value_real is not None:
                        v=eval_func(vals[item]) if type(vals[item]) is str else vals[item]
                        vals[item]=grid_value_real+v.imag*1.0J
                    if grid_value_imag is not None:
                        v=eval_func(vals[item]) if type(vals[item]) is str else vals[item]
                        vals[item]=v.real+grid_value_imag*1.0J
                    editable[item]=max(grid_parameters.get_fit_state_by_name(func_name+'real'),
                                       grid_parameters.get_fit_state_by_name(func_name+'imag'))

                items.append((item, value))
                pars.append(item)

                # Check if the parameter is in the grid and in that case set it as uneditable
                # func_name = obj_name + '.' + _set_func_prefix + item.capitalize()
                # grid_value = grid_parameters.get_value_by_name(func_name)
                # editable[item] = grid_parameters.get_fit_state_by_name(func_name)

            try:
                groups=self.model.LayerGroups
            except Exception:
                groups=False
            try:
                units=self.model.LayerUnits
            except Exception:
                units=False

            dlg=ValidateFitDialog(self, pars, vals, validators,
                                  title='Layer Editor', groups=groups,
                                  units=units, editable_pars=editable)

            if dlg.ShowModal()==wx.ID_OK:
                vals=dlg.GetValues()
                states=dlg.GetStates()
                for par in list(self.model.LayerParameters.keys()):
                    if not states[par]:
                        setattr(sel, par, vals[par])
                    if editable[par]!=states[par]:
                        value=eval_func(vals[par])

                        if type(self.model.LayerParameters[par]) is complex:
                            # print type(value)
                            func_name=obj_name+'.'+_set_func_prefix+par.capitalize()+'real'
                            val=value.real
                            minval=min(val*(1-self.variable_span), val*(1+self.variable_span))
                            maxval=max(val*(1-self.variable_span), val*(1+self.variable_span))
                            grid_parameters.set_fit_state_by_name(func_name, val, states[par], minval, maxval)
                            val=value.imag
                            minval=min(val*(1-self.variable_span), val*(1+self.variable_span))
                            maxval=max(val*(1-self.variable_span), val*(1+self.variable_span))
                            func_name=obj_name+'.'+_set_func_prefix+par.capitalize()+'imag'
                            grid_parameters.set_fit_state_by_name(func_name, val, states[par], minval, maxval)
                        else:
                            val=value
                            minval=min(val*(1-self.variable_span), val*(1+self.variable_span))
                            maxval=max(val*(1-self.variable_span), val*(1+self.variable_span))
                            func_name=obj_name+'.'+_set_func_prefix+par.capitalize()
                            grid_parameters.set_fit_state_by_name(func_name, value, states[par], minval, maxval)

                        # Does not seem to be necessary
                        self.plugin.parent.paramter_grid.SetParameters(grid_parameters)
                sl=self.sampleh.getStringList()
            dlg.Destroy()

        else:
            # The selected item is a Stack
            for item in list(self.model.StackParameters.keys()):
                if item!='Layers':
                    value=getattr(sel, item)
                    if isinstance(value, float):
                        validators[item]=FloatObjectValidator(eval_func, alt_types=[self.model.Stack])
                    else:
                        validators[item]=TextObjectValidator()
                    items.append((item, value))
                    pars.append(item)
                    vals[item]=value

                    # Check if the parameter is in the grid and in that case set it as uneditable
                    func_name=obj_name+'.'+_set_func_prefix+item.capitalize()
                    grid_value=grid_parameters.get_value_by_name(func_name)
                    editable[item]=grid_parameters.get_fit_state_by_name(func_name)
                    if grid_value is not None:
                        vals[item]=grid_value

            try:
                groups=self.model.StackGroups
            except Exception:
                groups=False
            try:
                units=self.model.StackUnits
            except Exception:
                units=False

            dlg=ValidateFitDialog(self, pars, vals, validators,
                                  title='Layer Editor', groups=groups,
                                  units=units, editable_pars=editable)
            if dlg.ShowModal()==wx.ID_OK:
                vals=dlg.GetValues()
                states=dlg.GetStates()
                for par in pars:
                    if not states[par]:
                        setattr(sel, par, vals[par])
                    if editable[par]!=states[par]:
                        value=eval_func(vals[par])
                        minval=min(value*(1-self.variable_span), value*(1+self.variable_span))
                        maxval=max(value*(1-self.variable_span), value*(1+self.variable_span))
                        func_name=obj_name+'.'+_set_func_prefix+par.capitalize()
                        grid_parameters.set_fit_state_by_name(func_name, value, states[par], minval, maxval)
                        # Does not seem to be necessary
                        self.plugin.parent.paramter_grid.SetParameters(grid_parameters)
                sl=self.sampleh.getStringList()

            dlg.Destroy()

        if sl:
            self.Update()

class DataParameterPanel(wx.Panel):
    ''' Widget that defines parameters coupling and different parameters
    for different data sets.
    '''

    def __init__(self, parent, plugin):
        wx.Panel.__init__(self, parent)
        self.plugin=plugin
        boxver=wx.BoxSizer(wx.VERTICAL)
        # Indention for a command - used to seperate commands and data
        self.command_indent='<pre>   '
        self.script_update_func=None
        self.parameterlist=[]

        self.toolbar=wx.ToolBar(self, style=wx.TB_FLAT | wx.TB_HORIZONTAL)
        self.do_toolbar()
        boxver.Add((-1, 2))
        boxver.Add(self.toolbar, proportion=0, flag=wx.EXPAND, border=1)
        boxver.Add((-1, 2))

        self.listbox=MyHtmlListBox(self, -1, style=wx.BORDER_SUNKEN)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.Edit, self.listbox)
        boxver.Add(self.listbox, 1, wx.EXPAND)

        self.SetSizer(boxver)

    def do_toolbar(self):
        dpi_scale_factor=wx.GetApp().dpi_scale_factor
        tb_bmp_size=int(dpi_scale_factor*20)

        button_names=['Insert', 'Delete', 'User Variables']
        button_images=[wx.Bitmap(images.add.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                       wx.Bitmap(images.delete.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                       wx.Bitmap(images.custom_parameters.GetImage().Scale(tb_bmp_size, tb_bmp_size))]
        callbacks=[self.Insert, self.Delete, self.EditPars]
        tooltips=['Insert a command', 'Delete command', 'Edit user variables']

        for i in range(len(button_names)):
            newid=wx.NewId()
            self.toolbar.AddTool(newid, label=button_names[i],
                                 bitmap=button_images[i],
                                 shortHelp=tooltips[i])
            self.Bind(wx.EVT_TOOL, callbacks[i], id=newid)
        self.toolbar.Realize()

    def onsimulate(self, event):
        """ Function to simulate the model.

        :return:
        """
        self.plugin.parent.eh_tb_simulate(event)

    def SetDataList(self, datalist):
        '''SetDataList(self, datalist) --> None
        
        Sets the name of the available data sets
        '''
        self.datalist=datalist

    def GetDataList(self):
        '''SetDataList(self) --> list
        
        Retrives the data list
        '''
        return self.datalist

    def SetParameterList(self, parameterlist):
        '''SetParameterList(self, parameterlist) --> None
        
        Sets the code list for all definition of custom parameters
        '''
        self.parameterlist=parameterlist

    def GetParameterList(self):
        '''SetParameterList(self) --> list
        
        Retrives the parameter list
        '''
        return self.parameterlist

    def SetExpressionList(self, expressionlist):
        '''SetExpressionList(expressionlist) --> None
        
        Sets the expression list, should be a 2D list with the 
        one list for each item in datalist
        '''
        if len(expressionlist)!=len(self.datalist):
            raise ValueError('The list of expression has to have the'+ \
                             ' same length as the data list')
        self.expressionlist=expressionlist

    def GetExpressionList(self):
        '''GetExpressionList(self) --> expression list
        
        Returns the expressionlist
        '''
        return self.expressionlist

    def SetSimArgs(self, sim_funcs, insts, args):
        '''SetSimArgs(self, sim_func, inst, args) --> None
        
        Sets the current simulation function for each data set 
        their instruments and arguments
        sim_func: A list of simulation names
        inst: A list of instrument names
        args: A list of argument list for the sim_func's each argument should be a string.
        '''
        if len(sim_funcs)!=len(self.datalist):
            raise ValueError('The list of sim_funcs has to have the'+ \
                             ' same length as the data list')
        if len(insts)!=len(self.datalist):
            raise ValueError('The list of insts has to have the'+ \
                             ' same length as the data list')
        if len(args)!=len(self.datalist):
            raise ValueError('The list of args has to have the'+ \
                             ' same length as the data list')
        self.sim_funcs=sim_funcs[:]
        self.insts=insts[:]
        self.args=args[:]

    def GetSimArgs(self):
        '''GetSimArgs(self) --> (sim_funcs, insts, args)
        
        See SetSimArgs for a description of the parameters
        '''
        return self.sim_funcs, self.insts, self.args

    def AppendSim(self, sim_func, inst, args):
        '''AppendSim(self, sim_func, inst, args) --> None

        Appends a simultion to the Panel lists
        '''
        self.sim_funcs.append(sim_func)
        self.insts.append(inst)
        self.args.append(args)

    def InstrumentNameChange(self, old_name, new_name):
        '''OnInstrumentNameChange --> None
        
        Exchanges old_name to new name in the simulations.'''

        for i in range(len(self.insts)):
            if self.insts[i]==old_name:
                self.insts[i]=new_name
        self.update_listbox()

    def SetUpdateScriptFunc(self, func):
        '''SetUpdateScriptFunc(self, func) --> None
        
        Sets the function to be called when the script needs to be updated.
        will only be called as func(event)
        '''
        self.script_update_func=func

    def UpdateListbox(self, update_script=True):
        '''Update(self) --> None
        
        Update the listbox and runs the callback script_update_func
        '''
        self.update_listbox()
        if self.script_update_func and update_script:
            self.script_update_func(None)
        self.Refresh()

    def update_listbox(self):
        '''update_listbox(self) --> None
     
         updates the listbox.
        '''
        list_strings=[]
        for i in range(len(self.datalist)):
            str_arg=', '.join(self.args[i])
            list_strings.append('<code><b>%s</b>: %s(%s, %s)</code>'
                                ' \n'%(self.datalist[i],
                                       self.sim_funcs[i],
                                       str_arg, self.insts[i]))
            for item in self.expressionlist[i]:
                list_strings.append(self.command_indent+'%s</pre>'%item)

        self.listbox.SetItemList(list_strings)

    def get_expression_position(self):
        '''get_expression_position(self) --> (dataitem, expression)
        
        Finds the position in the expression list for a certain item.
        return -1 if it can not be found.
        '''
        index=self.listbox.GetSelection()

        if index==wx.NOT_FOUND:
            return -1, -1

        dataindex=-1
        itemindex=-1
        listindex=-1
        for i in range(len(self.datalist)):
            dataindex+=1
            listindex+=1
            itemindex=-1
            if listindex>=index:
                return dataindex, itemindex
            for item in self.expressionlist[i]:
                listindex+=1
                itemindex+=1
                if listindex>=index:
                    return dataindex, itemindex

        # If all other things fail...
        return -1, -1

    def Edit(self, event):
        '''Edit(self, event) --> None
        
        Edits an entry in the list
        '''
        data_pos, exp_pos=self.get_expression_position()
        if exp_pos!=-1 and data_pos!=-1:
            # Editing the expressions for variables
            list_item=self.expressionlist[data_pos][exp_pos]
            dlg=ParameterExpressionDialog(self, self.plugin.GetModel(),
                                          list_item, sim_func=self.onsimulate)
            if dlg.ShowModal()==wx.ID_OK:
                exp=dlg.GetExpression()
                self.expressionlist[data_pos][exp_pos]=exp
                self.UpdateListbox()
        if exp_pos==-1 and data_pos!=-1:
            # Editing the simulation function and its arguments
            dlg=SimulationExpressionDialog(self, self.plugin.GetModel(),
                                           self.plugin.sample_widget.instruments,
                                           self.sim_funcs[data_pos],
                                           self.args[data_pos],
                                           self.insts[data_pos], data_pos)
            if dlg.ShowModal()==wx.ID_OK:
                self.args[data_pos]=dlg.GetExpressions()
                self.insts[data_pos]=dlg.GetInstrument()
                self.sim_funcs[data_pos]=dlg.GetSim()
                self.UpdateListbox()

    def Insert(self, event):
        ''' Insert(self, event) --> None
        
        Inserts a new operations
        '''
        data_pos, exp_pos=self.get_expression_position()
        if data_pos!=-1:
            dlg=ParameterExpressionDialog(self, self.plugin.GetModel(), sim_func=self.onsimulate)
            if dlg.ShowModal()==wx.ID_OK:
                exp=dlg.GetExpression()
                if exp_pos==-1:
                    self.expressionlist[data_pos].insert(0, exp)
                else:
                    self.expressionlist[data_pos].insert(exp_pos, exp)
                self.UpdateListbox()

    def Delete(self, event):
        '''Delete(self, event) --> None
        
        Deletes an operation
        '''
        data_pos, exp_pos=self.get_expression_position()
        if exp_pos!=-1 and data_pos!=-1:
            self.expressionlist[data_pos].pop(exp_pos)
            self.UpdateListbox()

    def MoveUp(self, event):
        '''MoveUp(self, event) --> None
        
        Move an operation up
        '''
        pass

    def MoveDown(self, event):
        '''MoveDown(self, event) --> None
        
        Moves an operation down
        '''
        pass

    def EditPars(self, event):
        ''' EditPars(self, event) --> None
        
        Creates a new parameter
        '''
        dlg=EditCustomParameters(self, self.plugin.GetModel(),
                                 self.parameterlist)
        if dlg.ShowModal()==wx.ID_OK:
            self.parameterlist=dlg.GetLines()
            self.UpdateListbox()
        dlg.Destroy()

    def OnDataChanged(self, event):
        '''OnDataChanged(self, event) --> None
        
        Updated the data list
        '''
        self.UpdateListbox(update_script=False)

class EditCustomParameters(wx.Dialog):
    def __init__(self, parent, model, lines):
        wx.Dialog.__init__(self, parent, -1, 'Custom parameter editor')
        self.SetAutoLayout(True)
        self.model=model
        self.lines=lines
        self.var_name='cp'

        sizer=wx.BoxSizer(wx.VERTICAL)
        name_ctrl_sizer=wx.GridBagSizer(2, 3)

        col_labels=['Name', 'Value', 'Sigma (for systematic error)']

        for item, index in zip(col_labels, list(range(len(col_labels)))):
            label=wx.StaticText(self, -1, item)
            name_ctrl_sizer.Add(label, (0, index), flag=wx.ALIGN_LEFT, border=5)

        self.name_ctrl=wx.TextCtrl(self, -1, size=(120, -1))
        name_ctrl_sizer.Add(self.name_ctrl, (1, 0),
                            flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)
        self.value_ctrl=wx.TextCtrl(self, -1, size=(120, -1))
        name_ctrl_sizer.Add(self.value_ctrl, (1, 1),
                            flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)
        self.error_ctrl=wx.TextCtrl(self, -1, size=(120, -1))
        name_ctrl_sizer.Add(self.error_ctrl, (1, 2),
                            flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)
        self.add_button=wx.Button(self, id=wx.ID_ANY, label='Add')
        name_ctrl_sizer.Add(self.add_button, (1, 3),
                            flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)
        sizer.Add(name_ctrl_sizer)
        self.Bind(wx.EVT_BUTTON, self.OnAdd, self.add_button)

        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.RIGHT | wx.TOP, 5)

        self.listbox=MyHtmlListBox(self, -1, size=(-1, 150),
                                   style=wx.BORDER_SUNKEN)
        self.listbox.SetItemList(self.lines)
        sizer.Add(self.listbox, 1, wx.GROW | wx.ALL, 10)

        self.delete_button=wx.Button(self, id=wx.ID_ANY, label='Delete')
        sizer.Add(self.delete_button, 0, wx.CENTRE, 0)
        self.Bind(wx.EVT_BUTTON, self.OnDelete, self.delete_button)

        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, id=wx.ID_OK)
        # okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        button_sizer.AddButton(wx.Button(self, id=wx.ID_CANCEL))
        button_sizer.Realize()
        self.Bind(wx.EVT_BUTTON, self.OnApply, okay_button)

        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.RIGHT | wx.TOP, 5)

        sizer.Add(button_sizer, 0, wx.ALIGN_RIGHT, 5)
        self.SetSizer(sizer)
        sizer.Fit(self)
        self.Layout()

    def OnApply(self, event):
        '''OnApply(self, event) --> None
        
        Callback for ok button click or apply button
        '''
        event.Skip()

    def OnAdd(self, event):
        '''OnAdd(self, event) --> None
        
        Callback for adding an entry
        '''
        sigma=self.error_ctrl.GetValue()
        if sigma.strip()=='':
            line='%s.new_var(\'%s\', %s)'%(self.var_name,
                                           self.name_ctrl.GetValue(), self.value_ctrl.GetValue())
        else:
            line='%s.new_sys_err(\'%s\', %s, %s)'%(self.var_name, self.name_ctrl.GetValue(),
                                                   self.value_ctrl.GetValue(), sigma)
        try:
            self.model.eval_in_model(line)
        except Exception as e:
            result='Could not evaluate the expression. The python error'+ \
                   'is: \n'+e.__repr__()
            dlg=wx.MessageDialog(self, result, 'Error in expression',
                                 wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
        else:
            self.lines.append(line)
            self.listbox.SetItemList(self.lines)

    def OnDelete(self, event):
        '''OnDelete(self, event) --> None
        
        Callback for deleting an entry
        '''
        result='Do you want to delete the expression?\n'+ \
               'Remember to check if parameter is used elsewhere!'
        dlg=wx.MessageDialog(self, result, 'Delete expression?',
                             wx.YES_NO | wx.NO_DEFAULT | wx.ICON_INFORMATION)
        if dlg.ShowModal()==wx.ID_YES:
            self.lines.pop(self.listbox.GetSelection())
            self.listbox.SetItemList(self.lines)
        dlg.Destroy()

    def GetLines(self):
        '''GetLines(self) --> uservars lines [list]
        
        Returns the list user variables.
        '''
        return self.lines

class SimulationExpressionDialog(wx.Dialog):
    '''A dialog to edit the Simulation expression
    '''

    def __init__(self, parent, model, instruments, sim_func, arguments, inst_name,
                 data_index):
        '''Creates a SimualtionExpressionDialog. 
        
        model - a Model object.
        instruments - a dictionary of possible instruments
        arguments - the arguments to the simulation function, a list of strings.
        sim_func - a string of the simulation function name.
        inst_name - the name of the current instrument.
        data_index - an integer for the current data index.
        '''

        self.model=model
        self.instruments=instruments
        self.available_sim_funcs=list(self.model.eval_in_model('model.SimulationFunctions.keys()'))
        self.data_index=data_index

        # Do the layout of the dialog
        wx.Dialog.__init__(self, parent, -1, 'Simulation editor')
        self.SetAutoLayout(True)

        # Find out the maximum number of arguments to the available sim_funcs
        max_val=-1
        self.sim_args={}
        self.sim_defaults={}
        for func in self.available_sim_funcs:
            doc=self.model.eval_in_model('model.SimulationFunctions'
                                         '["%s"].__doc__'%func)
            doc_lines=find_code_segment(doc, 'Parameters').splitlines()
            max_val=max(len(doc_lines), max_val)
            args=[]
            defaults=[]
            for line in doc_lines:
                items=line.lstrip().rstrip().split(' ')
                args.append(items[0])
                defaults.append(items[1].replace('data', 'd'))
            self.sim_args[func]=args
            self.sim_defaults[func]=defaults

        expressions={'Instrument': inst_name}
        for arg_name, arg in zip(self.sim_args[sim_func], arguments):
            expressions[arg_name]=arg

        if max_val<0:
            raise ValueError('Wrongly formatted function docs for the simulation functions')

        gbs=wx.GridBagSizer(2, max_val)

        # Creating the column labels
        col_labels=['Simulation', 'Instrument']
        [col_labels.append(arg) for arg in self.sim_args[sim_func] if not arg in col_labels]
        self.labels=[]
        self.arg_controls=[]
        for index in range(2+max_val):
            label=wx.StaticText(self, -1, '')
            gbs.Add(label, (0, index), flag=wx.ALIGN_LEFT, border=5)
            self.labels.append(label)
            # If the expression is not an instrument or simulation function
            if index>1:
                exp_ctrl=wx.TextCtrl(self, -1, size=(100, -1))
                gbs.Add(exp_ctrl, (1, index),
                        flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)
                self.arg_controls.append(exp_ctrl)

        for item, label in zip(col_labels[:2], self.labels[:2]):
            label.SetLabel(item)
        # Creating the text boxes for the arguments
        # Setting the text in the column labels and text controls
        for item, label, arg_ctrl in zip(col_labels[2:],
                                         self.labels[2:],
                                         self.arg_controls):
            label.SetLabel(item)
            arg_ctrl.SetValue(expressions[item])
            arg_ctrl.SetEditable(True)

        for i in range(len(col_labels)-2, len(self.arg_controls)):
            self.arg_controls[i].SetEditable(False)
            # self.arg_controls[i].Show(False)
            # self.arg_controls[i].SetValue('NA')

        # Creating the controls
        # Simulation choice control
        self.sim_choice=wx.Choice(self, -1,
                                  choices=self.available_sim_funcs)
        self.Bind(wx.EVT_CHOICE, self.on_sim_change, self.sim_choice)
        self.sim_choice.SetSelection(self.available_sim_funcs.index(sim_func))
        gbs.Add(self.sim_choice, (1, 0),
                flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)

        # Instrument choice control
        self.inst_choice=wx.Choice(self, -1,
                                   choices=list(self.instruments.keys()))
        # self.Bind(wx.EVT_CHOICE, self.on_inst_change, self.inst_choice)
        self.inst_choice.SetSelection(list(self.instruments.keys()).index(expressions['Instrument']))
        gbs.Add(self.inst_choice, (1, 1),
                flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)

        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))

        button_sizer.Realize()
        self.Bind(wx.EVT_BUTTON, self.on_ok_button, okay_button)

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add(gbs, 1, wx.GROW | wx.ALL, 10)
        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.RIGHT | wx.TOP, 5)
        sizer.Add((-1, 5))

        sizer.Add(button_sizer, 0, wx.ALIGN_RIGHT, 5)
        sizer.Add((-1, 5))
        self.SetSizer(sizer)
        sizer.Fit(self)
        self.Layout()

    def on_sim_change(self, evt):
        '''Callback for changing the choice widget for the different simulations.
        '''
        new_sim=self.sim_choice.GetStringSelection()
        # Update the column labels
        new_labels=[]
        for label, arg_name in zip(self.labels[2:], self.sim_args[new_sim]):
            new_labels.append(label.GetLabel()!=arg_name)
            label.SetLabel(arg_name)
        # Clear the remaining column labels
        for label in self.labels[len(self.sim_args[new_sim])+2:]:
            label.SetLabel('')

        # Update the text controls - if needed
        for i in range(len(self.sim_args[new_sim])):
            # if new_labels[i]:
            if True:
                self.arg_controls[i].SetValue(self.sim_defaults[new_sim][i])
            self.arg_controls[i].SetEditable(True)
            # self.arg_controls[i].Show(True)
        # Hide and clear the remaining text controls
        for ctrl in self.arg_controls[len(self.sim_args[new_sim]):]:
            ctrl.SetEditable(False)
            ctrl.SetValue('')

            # ctrl.Show(False)

    def on_ok_button(self, event):
        '''Callback for pressing the ok button in the dialog'''
        expressions=self.GetExpressions()
        # Hack to get it working with d = data[0]
        exec('d = data[%d]'%self.data_index, self.model.script_module.__dict__)
        for exp in expressions:
            try:
                self.model.eval_in_model(exp)
            except Exception as e:
                result=('Could not evaluate expression:\n%s.\n'%exp+
                        ' The python error is: \n'+e.__repr__())
                dlg=wx.MessageDialog(self, result, 'Error in expression',
                                     wx.OK | wx.ICON_WARNING)
                dlg.ShowModal()
                dlg.Destroy()
            else:
                event.Skip()

    def GetExpressions(self):
        ''' Returns the current expressions in the dialog box '''
        return [ctrl.GetValue() for ctrl in self.arg_controls
                if ctrl.IsEditable()]

    def GetInstrument(self):
        ''' Returns the selected instrument, a string'''
        return self.inst_choice.GetStringSelection()

    def GetSim(self):
        ''' Returns the selected simulation, a string'''
        return self.sim_choice.GetStringSelection()

class ParameterExpressionDialog(wx.Dialog):
    ''' A dialog for setting parameters for fitting
    '''

    def __init__(self, parent, model, expression=None, sim_func=None):
        wx.Dialog.__init__(self, parent, -1, 'Parameter editor')
        self.SetAutoLayout(True)
        self.model=model
        self.sim_func=sim_func

        gbs=wx.GridBagSizer(2, 3)

        col_labels=['Object', 'Parameter', 'Expression']

        for item, index in zip(col_labels, list(range(len(col_labels)))):
            label=wx.StaticText(self, -1, item)
            gbs.Add(label, (0, index), flag=wx.ALIGN_LEFT, border=5)

        # Get the objects that should be in the choiceboxes
        par_dict=model.get_possible_parameters()
        objlist=[]
        funclist=[]
        for cl in par_dict:
            obj_dict=par_dict[cl]
            for obj in obj_dict:
                objlist.append(obj)
                funclist.append(obj_dict[obj])

        self.objlist=objlist
        self.funclist=funclist
        self.obj_choice=wx.Choice(self, -1, choices=objlist)
        self.Bind(wx.EVT_CHOICE, self.on_obj_change, self.obj_choice)

        self.func_choice=wx.Choice(self, -1)
        # This will init func_choice
        self.obj_choice.SetSelection(0)

        gbs.Add(self.obj_choice, (1, 0),
                flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)
        gbs.Add(self.func_choice, (1, 1),
                flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)

        exp_right=''
        if expression:
            p=expression.find('(')
            exp_left=expression[:p]
            obj=exp_left.split('.')[0]
            func=exp_left.split('.')[1]
            exp_right=expression[p+1:-1]
            obj_pos=[i for i in range(len(objlist)) if objlist[i]==obj]
            if len(obj_pos)>0:
                self.obj_choice.SetSelection(obj_pos[0])
                self.on_obj_change(None)
                func_pos=[i for i in range(len(funclist[obj_pos[0]])) \
                          if funclist[obj_pos[0]][i]==func]
                if len(func_pos)>0:
                    self.func_choice.SetSelection(func_pos[0])
                else:
                    raise ValueError('The function %s for object %s does not exist'%(func, obj))
            else:
                raise ValueError('The object %s does not exist'%obj)

        # self.expression_ctrl = wx.TextCtrl(self, -1, exp_right,\
        #                       size=(300, -1))

        self.expression_ctrl=ParameterExpressionCombo(par_dict, sim_func, self, -1, exp_right,
                                                      size=(300, -1))
        gbs.Add(self.expression_ctrl, (1, 2),
                flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, border=5)

        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        # apply_button = wx.Button(self, wx.ID_APPLY)
        # apply_button.SetDefault()
        # button_sizer.AddButton(apply_button)
        button_sizer.Realize()
        self.Bind(wx.EVT_BUTTON, self.OnApply, okay_button)
        # self.Bind(wx.EVT_BUTTON, self.OnApply, apply_button)

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add(gbs, 1, wx.GROW | wx.ALL, 10)
        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.RIGHT | wx.TOP, 5)
        sizer.Add((-1, 5))

        sizer.Add(button_sizer, 0, wx.ALIGN_RIGHT, 5)
        sizer.Add((-1, 5))
        self.SetSizer(sizer)
        sizer.Fit(self)
        self.Layout()

    def on_obj_change(self, event):
        '''on_obj_change(self, event) --> None
        
        On changing the object the funclist should be updated
        '''
        index=self.obj_choice.GetSelection()
        self.func_choice.SetItems(self.funclist[index])

    def OnApply(self, event):
        '''OnApply(self, event) --> None
        '''
        evalstring=self.GetExpression()
        try:
            self.model.eval_in_model(evalstring)
        except Exception as e:
            result='Could not evaluate the expression. The python'+ \
                   'is: \n'+e.__repr__()
            dlg=wx.MessageDialog(self, result, 'Error in expression',
                                 wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
        else:
            event.Skip()

    def GetExpression(self):
        '''GetExpression(self) --> expression
        
        Yields the string that has been edited in the dialog
        '''
        objstr=self.obj_choice.GetStringSelection()
        funcstr=self.func_choice.GetStringSelection()
        set_expression=self.expression_ctrl.GetValue()
        evalstring='%s.%s(%s)'%(objstr, funcstr, set_expression)

        return evalstring

class SamplePlotPanel(wx.Panel):
    ''' Widget for plotting the scattering length density of 
    a sample.
    '''

    def __init__(self, parent, plugin, id=-1, color=None, dpi=None
                 , style=wx.NO_FULL_REPAINT_ON_RESIZE, **kwargs):
        ''' Inits the plotpanel
        '''
        wx.Panel.__init__(self, parent)
        self.plot=PlotPanel(self, -1, color, dpi, style, **kwargs)
        self.plugin=plugin

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.plot, 1, wx.EXPAND | wx.GROW | wx.ALL)

        self.plot.update(None)
        self.plot.ax=self.plot.figure.add_subplot(111)
        box=self.plot.ax.get_position()
        self.plot.ax.set_position([box.x0, box.y0, box.width*0.95, box.height])
        self.plot.ax.set_autoscale_on(True)
        self.plot.update=self.Plot
        self.SetSizer(sizer)
        self.plot.ax.set_autoscale_on(False)
        self.plot_dict={}

    def SetZoom(self, active=False):
        return self.plot.SetZoom(active)

    def Plot(self):
        ''' Plot(self) --> None
        
        Plotting the sample Sample.
        '''
        colors=['b', 'r', 'g', 'c', 'm', 'y', 'k']
        model=self.plugin.GetModel().script_module
        # self.plot_dict = model.sample.SimSLD(None, model.inst)
        self.plot_dicts=[]
        self.plot.ax.lines=[]
        self.plot.ax.clear()
        i=0
        data=self.plugin.GetModel().get_data()
        sld_units=[]

        if self.plugin.sim_returns_sld and model._sim:
            # New style sim function with one sld for each simulation
            self.plot_dicts=model.SLD
            for sim in range(len(self.plot_dicts)):
                if data[sim].show:
                    for key in self.plot_dicts[sim]:
                        try:
                            if key in ['z', 'SLD unit'] or (self.plot_dicts[0][key]==0).all():
                                # skip lines that are all zero to keep legend cleaner
                                continue
                        except KeyError:
                            pass
                        is_imag=key[:2]=='Im' or key[:4]=='imag'
                        if (is_imag and self.plugin.show_imag_sld) or not is_imag:
                            label=data[sim].name+'\n'+key
                            self.plot.ax.plot(self.plot_dicts[sim]['z'], self.plot_dicts[sim][key],
                                              colors[i%len(colors)], label=label)

                            if 'SLD unit' in self.plot_dicts[sim]:
                                if not self.plot_dicts[sim]['SLD unit'] in sld_units:
                                    sld_units.append(self.plot_dicts[sim]['SLD unit'])
                            i+=1
        else:
            # Old style plotting just one sld
            if self.plugin.GetModel().compiled:
                try:
                    sample=model.sample
                except AttributeError:
                    iprint("Warning: Could not locate the sample in the model")
                    return
                plot_dict=sample.SimSLD(None, None, model.inst)
                self.plot_dicts=[plot_dict]
                for key in self.plot_dicts[0]:
                    if key in ['z', 'SLD unit'] or (self.plot_dicts[0][key]==0).all():
                        # skip lines that are all zero to keep legend cleaner
                        continue
                    is_imag=key[:2]=='Im' or key[:4]=='imag'
                    if (is_imag and self.plugin.show_imag_sld) or not is_imag:
                        label=key
                        self.plot.ax.plot(self.plot_dicts[0]['z'], self.plot_dicts[0][key],
                                          colors[i%len(colors)], label=label)

                        if 'SLD unit' in self.plot_dicts[0]:
                            if not self.plot_dicts[0]['SLD unit'] in sld_units:
                                sld_units.append(self.plot_dicts[0]['SLD unit'])
                        i+=1

        if i>0:
            self.plot.ax.legend(loc='upper right',  # bbox_to_anchor=(1, 0.5),
                                framealpha=0.5,
                                fontsize="small", ncol=1)

            sld_unit=', '.join(sld_units)
            self.plot.ax.yaxis.label.set_text('$\mathrm{\mathsf{SLD\,[%s]}}$'%sld_unit)
            # if self.plot_dict.has_key('SLD unit'):
            #    self.plot.ax.yaxis.label.set_text('$\mathrm{\mathsf{SLD\,[%s]}}$'%(sld_unit))
            self.plot.ax.xaxis.label.set_text('$\mathrm{\mathsf{ z\,[\AA]}}$')
            wx.CallAfter(self.plot.flush_plot)
            self.plot.AutoScale()

    def SavePlotData(self, filename):
        ''' Save all the SLD profiles to file with filename.'''
        # Check so that there are a simulation to save
        try:
            self.plot_dicts
        except:
            self.plugin.ShowWarningDialog('No SLD data to save.'
                                          ' Simulate the model first and then save.')
            return
        base, ext=os.path.splitext(filename)
        if ext=='':
            ext='.dat'
        data=self.plugin.GetModel().get_data()
        for sim in range(len(self.plot_dicts)):
            new_filename=(base+'%03d'%sim+ext)
            save_array=np.array([self.plot_dicts[sim]['z']])
            header='z\t'
            for key in self.plot_dicts[sim]:
                if key!='z' and key!='SLD unit':
                    save_array=np.r_[save_array, [self.plot_dicts[sim][key]]]
                    header+=key+'\t'
            with open(new_filename, 'w') as f:
                f.write("# File exported from GenX's Reflectivity plugin\n")
                f.write("# File created: %s\n"%time.ctime())
                f.write("# Simulated SLD for data set: %s\n"%data[sim].name)
                f.write("# Headers: \n")
                f.write('#'+header+'\n')
                np.savetxt(f, save_array.transpose())

    def ReadConfig(self):
        '''ReadConfig(self) --> None

        Reads in the config file
        '''
        return self.plot.ReadConfig()

    def GetZoom(self):
        return self.plot.GetZoom()

    def GetYScale(self):
        '''GetYScale(self) --> String

        Returns the current y-scale in use. Currently the string
        'log' or 'linear'. If the axes does not exist it returns None.
        '''
        if self.plot.ax:
            return self.plot.ax.get_yscale()
        else:
            return None

    def GetXScale(self):
        '''GetXScale(self) --> String

        Returns the current x-scale in use. Currently the string
        'log' or 'linear'. If the axes does not exist it returns None.
        '''
        if self.plot.ax:
            return self.plot.ax.get_xscale()
        else:
            return None

class Plugin(framework.Template, SampleBuilder):
    previous_xaxis=None
    _last_script=None

    def __init__(self, parent):
        if 'SimpleReflectivity' in parent.plugin_control.plugin_handler.get_loaded_plugins():
            parent.plugin_control.UnLoadPlugin_by_Name('SimpleReflectivity')
        framework.Template.__init__(self, parent)
        # self.parent = parent
        self.model_obj=self.GetModel()
        sample_panel=self.NewInputFolder('Sample')
        sample_sizer=wx.BoxSizer(wx.HORIZONTAL)
        sample_panel.SetSizer(sample_sizer)
        self.sample_widget=SamplePanel(sample_panel, self)
        sample_sizer.Add(self.sample_widget, 1, wx.EXPAND | wx.GROW | wx.ALL)
        sample_panel.Layout()

        simulation_panel=self.NewInputFolder('Simulations')
        simulation_sizer=wx.BoxSizer(wx.HORIZONTAL)
        simulation_panel.SetSizer(simulation_sizer)
        self.simulation_widget=DataParameterPanel(simulation_panel, self)
        simulation_sizer.Add(self.simulation_widget, 1, wx.EXPAND | wx.GROW | wx.ALL)
        simulation_panel.Layout()

        self.sample_widget.SetUpdateCallback(self.UpdateScript)
        self.simulation_widget.SetUpdateScriptFunc(self.UpdateScript)

        # Create the SLD plot
        sld_plot_panel=self.NewPlotFolder('SLD')
        sld_sizer=wx.BoxSizer(wx.HORIZONTAL)
        sld_plot_panel.SetSizer(sld_sizer)
        self.sld_plot=SamplePlotPanel(sld_plot_panel, self)
        sld_sizer.Add(self.sld_plot, 1, wx.EXPAND | wx.GROW | wx.ALL)
        sld_plot_panel.Layout()

        if self.model_obj.script!='':
            if self.model_obj.filename!='':
                iprint("Reflectivity plugin: Reading loaded model")
                try:
                    self.ReadModel()
                except Exception as e:
                    iprint("Reflectivity plugin model could not be read.")
                    self.Remove()
                    raise e
            else:
                try:
                    self.ReadModel()
                except:
                    iprint("Reflectivity plugin: Creating new model")
                    self.CreateNewModel()
        else:
            iprint("Reflectivity plugin: Creating new model")
            self.CreateNewModel()

        # Create a menu for handling the plugin
        menu=self.NewMenu('Reflec')
        self.mb_export_sld=wx.MenuItem(menu, wx.NewId(),
                                       "Export SLD...",
                                       "Export the SLD to a ASCII file",
                                       wx.ITEM_NORMAL)
        menu.Append(self.mb_export_sld)
        self.mb_show_imag_sld=wx.MenuItem(menu, wx.NewId(),
                                          "Show Im SLD",
                                          "Toggles showing the imaginary part of the SLD",
                                          wx.ITEM_CHECK)
        menu.Append(self.mb_show_imag_sld)
        self.mb_show_imag_sld.Check(False)
        self.show_imag_sld=self.mb_show_imag_sld.IsChecked()
        self.mb_autoupdate_sld=wx.MenuItem(menu, wx.NewId(),
                                           "Autoupdate SLD",
                                           "Toggles autoupdating the SLD during fitting",
                                           wx.ITEM_CHECK)
        menu.Append(self.mb_autoupdate_sld)
        self.mb_autoupdate_sld.Check(False)
        # self.mb_autoupdate_sld.SetCheckable(True)
        self.parent.Bind(wx.EVT_MENU, self.OnExportSLD, self.mb_export_sld)
        self.parent.Bind(wx.EVT_MENU, self.OnAutoUpdateSLD, self.mb_autoupdate_sld)
        self.parent.Bind(wx.EVT_MENU, self.OnShowImagSLD, self.mb_show_imag_sld)

        self.StatusMessage('Reflectivity plugin loaded')

    def SetModelScript(self, script):
        framework.Template.SetModelScript(self, script)
        self._last_script=script

    def InputPageChanged(self, pname):
        # check if the script was changed and in that case update the model
        if self.parent.script_editor.GetText()!=self._last_script:
            # update the script, this is done automatically when simulating by user
            self.model_obj.set_script(self.parent.script_editor.GetText())
            try:
                self.ReadModel(reevaluate=True)
            except Exception as e:
                self.ShowErrorDialog("Error in evaluation of model script, old model retained!")

    def UpdateScript(self, event):
        self.WriteModel()

    def OnAutoUpdateSLD(self, evt):
        # self.mb_autoupdate_sld.Check(not self.mb_autoupdate_sld.IsChecked())
        pass

    def OnShowImagSLD(self, evt):
        self.show_imag_sld=self.mb_show_imag_sld.IsChecked()
        self.sld_plot.Plot()

    def OnExportSLD(self, evt):
        dlg=wx.FileDialog(self.parent, message="Export SLD to ...",
                          defaultFile="",
                          wildcard="Dat File (*.dat)|*.dat",
                          style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                          )
        if dlg.ShowModal()==wx.ID_OK:
            fname=dlg.GetPath()
            result=True
            if os.path.exists(fname):
                filepath, filename=os.path.split(fname)
                result=self.ShowQuestionDialog('The file %s already exists.'
                                               ' Do'
                                               ' you wish to overwrite it?'
                                               %filename)
            if result:
                try:
                    self.sld_plot.SavePlotData(fname)
                except IOError as e:
                    self.ShowErrorDialog(e.__str__())
                except Exception as e:
                    outp=io.StringIO()
                    traceback.print_exc(200, outp)
                    val=outp.getvalue()
                    outp.close()
                    self.ShowErrorDialog('Could not save the file.'
                                         ' Python Error:\n%s'%(val,))
        dlg.Destroy()

    def OnNewModel(self, event):
        ''' Create a new model
        '''
        dlg=wx.SingleChoiceDialog(self.parent, 'Choose a model type to use',
                                  'Models', avail_models,
                                  wx.CHOICEDLG_STYLE
                                  )

        if dlg.ShowModal()==wx.ID_OK:
            self.CreateNewModel('models.%s'%dlg.GetStringSelection())
        dlg.Destroy()

    def OnDataChanged(self, event):
        ''' Take into account changes in data..
        '''
        if event.new_model:
            return

        if event.data_moved or event.deleted or event.new_data or event.name_change:
            names=[data_set.name for data_set in self.GetModel().get_data()]
            self.simulation_widget.SetDataList(names)

            expl=self.simulation_widget.GetExpressionList()

            if len(names)-len(expl)==1:
                # Data set has been added:
                expl.append([])
                self.insert_new_data_segment(len(expl)-1)

            sims, insts, args=self.simulation_widget.GetSimArgs()

            if event.deleted:
                pos=list(range(len(expl)))
                [self.remove_data_segment(pos[-index-1]) for index in \
                 range(len(event.position))]
                [expl.pop(index) for index in event.position]
                [sims.pop(index) for index in event.position]
                [insts.pop(index) for index in event.position]
                [args.pop(index) for index in event.position]
            elif event.data_moved:
                if event.up:
                    # Moving up
                    for pos in event.position:
                        tmp=expl.pop(pos)
                        expl.insert(pos-1, tmp)
                        tmp=sims.pop(pos)
                        sims.insert(pos-1, tmp)
                        tmp=insts.pop(pos)
                        insts.insert(pos-1, tmp)
                        tmp=args.pop(pos)
                        args.insert(pos-1, tmp)
                else:
                    # Moving down...
                    for pos in event.position:
                        tmp=expl.pop(pos)
                        expl.insert(pos+1, tmp)
                        tmp=sims.pop(pos)
                        sims.insert(pos+1, tmp)
                        tmp=insts.pop(pos)
                        insts.insert(pos+1, tmp)
                        tmp=args.pop(pos)
                        args.insert(pos+1, tmp)

            self.simulation_widget.SetSimArgs(sims, insts, args)
            self.simulation_widget.SetExpressionList(expl)

            # Check so we have not clicked on new model button
            if self.GetModel().script!='':
                self.WriteModel()
                self.simulation_widget.UpdateListbox()
                if event.name_change:
                    self.sld_plot.Plot()
            else:
                self.simulation_widget.UpdateListbox(update_script=True)
        else:
            if event.data_changed:
                self.sld_plot.Plot()

    def OnOpenModel(self, event):
        '''OnOpenModel(self, event) --> None
        
        Loads the sample into the plugin...
        '''

        self.ReadModel()

    def OnSimulate(self, event):
        '''OnSimulate(self, event) --> None
        
        Updates stuff after simulation
        '''
        # Calculate and update the sld plot
        wx.CallAfter(self.sld_plot.Plot)

    def OnFittingUpdate(self, event):
        '''OnSimulate(self, event) --> None
        
        Updates stuff during fitting
        '''
        # Calculate and update the sld plot
        if self.mb_autoupdate_sld.IsChecked():
            wx.CallAfter(self.sld_plot.Plot)

    def OnGridChange(self, event):
        """ Updates the simualtion panel when the grid changes

        :param event:
        :return:
        """
        self.sample_widget.Update(update_script=False)

    def InstrumentNameChange(self, old_name, new_name):
        '''OnInstrumentNameChange --> None
        
        Exchanges old_name to new name in the simulaitons.
        '''
        self.simulation_widget.InstrumentNameChange(old_name, new_name)

    def CreateNewModel(self, modelname='models.spec_nx'):
        '''Init the script in the model to yield the 
        correct script for initilization
        '''
        model_data=self.GetModel().get_data()
        nb_data_sets=len(model_data)

        script=self.GetNewModelScript(modelname, nb_data_sets)
        self.BuildNewModel(script)

        self.sample_widget.sampleh=self.sampleh
        self.sample_widget.model=self.model
        instrument=self.model.Instrument()
        self.sample_widget.SetInstrument({'inst': instrument})

        names=[data_set.name for data_set in model_data]
        self.simulation_widget.SetDataList(names)
        self.simulation_widget.SetParameterList([])
        # An empty list to the expression widget...
        self.simulation_widget.SetExpressionList([[] for item in names])
        self.simulation_widget.SetSimArgs(['Specular']*nb_data_sets,
                                          ['inst']*nb_data_sets,
                                          [['d.x'] for i in range(nb_data_sets)])
        self.simulation_widget.UpdateListbox(update_script=True)

        self.sample_widget.Update(update_script=True)
        # self.WriteModel()

    def WriteModel(self):
        parameter_list=self.simulation_widget.GetParameterList()
        sim_funcs, sim_insts, sim_args=self.simulation_widget.GetSimArgs()
        expression_list=self.simulation_widget.GetExpressionList()
        instruments=self.sample_widget.instruments

        self.write_model_script(sim_funcs, sim_insts, sim_args,
                                expression_list, parameter_list, instruments)
        try:
            self.SetXAxis(instruments[sim_insts[0]])
        except AttributeError:
            pass

    def SetXAxis(self, instrument):
        if self.previous_xaxis==instrument.coords:
            return
        coords=instrument.coords
        if coords in self.model.InstrumentUnits:
            newx='%s [%s]'%(coords,
                            self.model.InstrumentUnits[coords])
        else:
            newx=coords
        self.parent.plot_data.update_labels(newx)
        from genx import data
        if coords=='q':
            data.DataSet.simulation_params[0]=0.001
            data.DataSet.simulation_params[1]=0.601
        else:
            data.DataSet.simulation_params[0]=0.01
            data.DataSet.simulation_params[1]=6.01
        for ds in self.parent.model.data:
            ds.run_command()

    def AppendSim(self, sim_func, inst, args):
        self.simulation_widget.AppendSim(sim_func, inst, args)

    def ReadModel(self, reevaluate=False):
        '''ReadModel(self)  --> None
        
        Reads in the current model and locates layers and stacks
        and sample defined inside BEGIN Sample section.
        '''
        self.StatusMessage('Compiling the script...')
        try:
            self.CompileScript()
        except modellib.GenericError as e:
            self.ShowErrorDialog(str(e))
            self.StatusMessage('Error when compiling the script')
            return
        except Exception as e:
            outp=io.StringIO()
            traceback.print_exc(200, outp)
            val=outp.getvalue()
            outp.close()
            self.ShowErrorDialog(val)
            self.StatusMessage('Fatal Error - compling, Reflectivity')
            return
        self.StatusMessage('Script compiled!')

        self.StatusMessage('Trying to interpret the script...')

        instrument_names=self.find_instrument_names()

        if len(instrument_names)==0:
            self.ShowErrorDialog('Could not find any Instruments in the'+ \
                                 ' model script. Check the script.')
            self.StatusMessage('ERROR No Instruments in script')
            return

        if not 'inst' in instrument_names:
            self.ShowErrorDialog('Could not find the default'+
                                 ' Instrument, inst, in the'+
                                 ' model script. Check the script.')
            self.StatusMessage('ERROR No Instrument called inst in script')
            return

        sample_text=self.find_sample_section()

        if sample_text is None:
            self.ShowErrorDialog('Could not find the sample section'+ \
                                 ' in the model script.\n Can not load the sample in the editor.')
            self.StatusMessage('ERROR No sample section in script')
            return

        all_names, layers, stacks=self.find_layers_stacks(sample_text)

        if len(layers)==0:
            self.ShowErrorDialog('Could not find any Layers in the'+ \
                                 ' model script. Check the script.')
            self.StatusMessage('ERROR No Layers in script')
            return

        # Now its time to set all the parameters so that we have the strings
        # instead of the evaluated value - looks better
        for lay in layers:
            for par in lay[1].split(','):
                vars=par.split('=')
                exec('%s.%s = "%s"'%(lay[0], vars[0].strip(), vars[1].strip()), self.GetModel().script_module.__dict__)
        try:
            data_names, insts, sim_args, sim_exp, sim_funcs=self.find_sim_function_parameters()
        except LookupError:
            self.ShowErrorDialog('Could not locate all data sets in the'
                                 ' script. There should be %i datasets'%len(self.GetModel().get_data()))
            self.StatusMessage('ERROR No Layers in script')
            return

        uservars_lines=self.find_user_parameters()

        self.model=self.GetModel().script_module.model
        sample=self.GetModel().script_module.sample

        self.sampleh=SampleHandler(sample, all_names)
        self.sampleh.model=self.model
        self.sample_widget.sampleh=self.sampleh
        self.sample_widget.model=self.model
        instruments={}
        for name in instrument_names:
            instruments[name]=getattr(self.GetModel().script_module, name)
        self.sample_widget.SetInstrument(instruments)

        self.simulation_widget.SetDataList(data_names)
        self.simulation_widget.SetExpressionList(sim_exp)
        self.simulation_widget.SetParameterList(uservars_lines)

        self.simulation_widget.SetSimArgs(sim_funcs, insts, sim_args)

        self.sample_widget.Update(update_script=False)
        self.simulation_widget.UpdateListbox(update_script=False)
        # The code have a tendency to screw up the model slightly when compiling it - the sample will be connected
        # to the module therefore reset the compiled flag so that the model has to be recompiled before fitting.
        self.GetModel().compiled=False
        if reevaluate:
            self.StatusMessage('Model analyzed and plugin updated!')
        else:
            self.StatusMessage('New sample loaded to plugin!')
        self._last_script=self.model_obj.script

        # Setup the plot x-axis and simulation standard
        try:
            self.SetXAxis(self.sample_widget.instruments[instrument_names[0]])
        except AttributeError:
            pass

if __name__=='__main__':
    import models.interdiff as Model

    nSi=3.0
    Fe=Model.Layer(d=10, sigmar=3.0, n=1-2.247e-5+2.891e-6j)
    Si=Model.Layer(d=15, sigmar=3.0, n='nSi')
    sub=Model.Layer(sigmar=3.0, n=1-7.577e-6+1.756e-7j)
    amb=Model.Layer(n=1.0)
    stack=Model.Stack(Layers=[Fe, Si], Repetitions=20)
    stack2=Model.Stack(Layers=[Fe, Si])
    sample=Model.Sample(Stacks=[stack, stack2], Ambient=amb, Substrate=sub, eta_z=500.0, eta_x=100.0)
    iprint(sample)
    inst=Model.Instrument(Wavelength=1.54, Coordinates=1)
    s=['Amb', 'stack1', 'Fe1', 'Si1', 's2', 'Fe2', 'Si2', 'Sub']
    sh=SampleHandler(sample, s)
    sh.getStringList()

    class MyApp(wx.App):
        def OnInit(self):
            # wx.InitAllImageHandlers()
            frame=SamplePanel(None, -1, "Sample", sh)
            frame.Show(True)
            self.SetTopWindow(frame)
            return True

    iprint(Si.getN().__repr__())
    app=MyApp(0)
    app.MainLoop()



'''
===============================
:mod:`xrdml`  XRDML data loader
===============================

Loads the data from Philips XPert instrument.
'''

import numpy as np
from xml.dom.minidom import parseString

from ..data_loader_framework import Template
from ..utils import ShowWarningDialog

try:
    import wx
    from wx.lib.masked import NumCtrl
except ImportError:
    class void():
        pass

    wx=void()
    wx.Dialog=void

class Plugin(Template):
    wildcard='*.xrdml'

    def __init__(self, parent):
        Template.__init__(self, parent)
        self.x_col=0
        self.y_col=1
        self.e_col=2
        self.xe_col=-1
        self.comment='#'
        self.skip_rows=0
        self.delimiter=None

    def LoadData(self, dataset, filename):
        '''
        Loads the data from filename into the data_item_number.
        '''
        try:
            datasets=ReadXpert(filename)
        except Exception as e:
            import traceback
            ShowWarningDialog(self.parent, 'Could not load the file: '+ \
                              filename+' \nPlease check the format.\n\n Error in ReadXpert:\n'+
                              traceback.format_exc())
        else:
            dataset=datasets[0]
            dataset.x_raw=dataset[0]
            dataset.y_raw=dataset[1]
            dataset.error_raw=dataset[2]
            # no xerror values
            dataset.xerror_raw=dataset[0]*0.+0.001
            # Run the commands on the data - this also sets the x,y, error memebers
            # of that data item.
            dataset.run_command()

            # insert metadata into ORSO compatible fields
            dataset.meta['data_source']['experiment']['instrument']='XRDML'
            dataset.meta['data_source']['experiment']['probe']='xray'
            dataset.meta['data_source']['measurement']['scheme']='angle-dispersive'

    def SettingsDialog(self):
        '''
        This function should - if necessary implement a dialog box
        that allows the user set import settings for example.
        '''
        col_values={'y': self.y_col, 'x': self.x_col,
                    'y error': self.e_col, 'x error': self.xe_col}
        misc_values={'Comment': str(self.comment), 'Skip rows': self.skip_rows,
                     'Delimiter': str(self.delimiter)}
        dlg=SettingsDialog(self.parent, col_values, misc_values)
        if dlg.ShowModal()==wx.ID_OK:
            col_values=dlg.GetColumnValues()
            misc_values=dlg.GetMiscValues()
            self.y_col=col_values['y']
            self.x_col=col_values['x']
            self.e_col=col_values['y error']
            self.xe_col=col_values['x error']
            self.comment=misc_values['Comment']
            self.skip_rows=misc_values['Skip rows']
            self.delimiter=misc_values['Delimiter']
        dlg.Destroy()

class SettingsDialog(wx.Dialog):

    def __init__(self, parent, col_values, misc_values):
        wx.Dialog.__init__(self, parent, -1, 'Data loader settings')

        box_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Make the box for putting in the columns
        col_box=wx.StaticBox(self, -1, "Columns")
        col_box_sizer=wx.StaticBoxSizer(col_box, wx.VERTICAL)

        # col_values = {'y': 1,'x': 0,'y error': 1}
        col_grid=wx.GridBagSizer(len(col_values), 2)
        self.col_controls=col_values.copy()
        keys=list(col_values.keys())
        keys.sort()
        for i, name in enumerate(keys):
            text=wx.StaticText(self, -1, name+': ')
            control=wx.SpinCtrl(self)
            control.SetRange(-10, 100)
            control.SetValue(col_values[name])
            col_grid.Add(text, (i, 0),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            col_grid.Add(control, (i, 1),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            self.col_controls[name]=control

        col_box_sizer.Add(col_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        box_sizer.Add(col_box_sizer, 0, wx.ALL | wx.EXPAND, 5)

        col_box=wx.StaticBox(self, -1, "Misc")
        col_box_sizer=wx.StaticBoxSizer(col_box, wx.VERTICAL)

        # Lets add another box for comments and rows to skip
        # misc_values = {'Comment': '#', 'Skip rows': 0,'Delimiter': 'None'}
        col_grid=wx.GridBagSizer(len(misc_values), 2)
        self.misc_controls=misc_values.copy()
        keys=list(misc_values.keys())
        keys.sort()
        for i, name in enumerate(keys):
            text=wx.StaticText(self, -1, name+': ')
            if type(misc_values[name])==type(1):
                control=wx.SpinCtrl(self)
                control.SetRange(0, 100)
                control.SetValue(misc_values[name])
            else:
                control=wx.TextCtrl(self, value=misc_values[name],
                                    style=wx.EXPAND)
            col_grid.Add(text, (i, 0),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            col_grid.Add(control, (i, 1),
                         flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                         border=5)
            self.misc_controls[name]=control

        col_box_sizer.Add(col_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        box_sizer.Add(col_box_sizer, 0, wx.ALL | wx.EXPAND, 5)

        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        button_sizer.Realize()

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add(box_sizer, 1, wx.GROW, 20)
        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW, 30)

        sizer.Add(button_sizer, 0,
                  flag=wx.ALIGN_RIGHT, border=20)
        self.SetSizer(sizer)

        sizer.Fit(self)
        self.Layout()

    def GetColumnValues(self):
        values={}
        for key in self.col_controls:
            values[key]=self.col_controls[key].GetValue()
        return values

    def GetMiscValues(self):
        values={}
        for key in self.misc_controls:
            val=self.misc_controls[key].GetValue()
            if type(val)==type('') or type(val)==type(''):
                if val.lower()=='none':
                    val=None
            values[key]=val
        return values

def ReadXpert(file_name):
    '''
      Read the data of a philips X'Pert diffractometer file, exported as text files.
    '''
    raw_data=open(file_name, 'r').read()
    xml_data=parseString(raw_data).firstChild

    # retrieve data
    try:
        sample_name=xml_data.getElementsByTagName('sample')[0].getElementsByTagName('name')[0].firstChild.nodeValue
    except AttributeError:
        sample_name=file_name.rsplit('.', 1)[0]
    datasets=[]
    for xml_scan in xml_data.getElementsByTagName('xrdMeasurement')[0].getElementsByTagName('scan'):
        scan=xml_scan.getElementsByTagName('dataPoints')[0]

        moving_positions={}
        for motor in scan.getElementsByTagName('positions'):
            axis=motor.attributes['axis'].value
            if len(motor.getElementsByTagName('commonPosition'))==0:
                start=float(motor.getElementsByTagName('startPosition')[0].firstChild.nodeValue)
                end=float(motor.getElementsByTagName('endPosition')[0].firstChild.nodeValue)
                moving_positions[axis]=(start, end)

        try:
            atten_factors=scan.getElementsByTagName('beamAttenuationFactors')[0].firstChild.nodeValue
            atten_factors=list(map(float, atten_factors.split()))
            atten=np.array(atten_factors)
        except IndexError:
            atten=1.0
        time=float(scan.getElementsByTagName('commonCountingTime')[0].firstChild.nodeValue)
        data=scan.getElementsByTagName('intensities')[0].firstChild.nodeValue
        data=list(map(float, data.split()))
        I=np.array(data)
        dI=np.sqrt(I*atten)/atten
        I/=time
        dI/=time
        if '2Theta' in moving_positions:
            th=np.linspace(moving_positions['2Theta'][0],
                           moving_positions['2Theta'][1], len(data))/2.
        else:
            th=np.linspace(moving_positions['Omega'][0],
                           moving_positions['Omega'][1], len(data))
        Q=4.*np.pi/1.54*np.sin(th/180.*np.pi)
        datasets.append((Q, I, dI, sample_name))
    return datasets



'''I/O functions for GenX. 
These include loading of initilazation files.
Also included is the config object.
File started by: Matts Bjorck

$Rev::                                  $:  Revision of last commit
$Author::                               $:  Author of last commit
$Date::                                 $:  Date of last commit
'''
try:
    import configparser as CP
except ImportError:
    import configparser as CP
import io
import os, sys

import h5py
from .gui_logging import iprint

# Functions to save the gx files
# ==============================================================================


def save_file(fname, model, optimizer, config):
    """Saves objects model, optimiser and config into file fnmame

    :param fname: string, ending with .gx or .hgx
    :param model:
    :param optimizer:
    :param config:
    :return:
    """
    if fname.endswith('.gx'):
        save_gx(fname, model, optimizer, config)
    elif fname.endswith('.hgx'):
        save_hgx(fname, model, optimizer, config)
    else:
        raise IOError('Wrong file ending, should be .gx or .hgx')

    model.filename=os.path.abspath(fname)
    model.saved=True

def load_file(fname, model, optimizer, config):
    """Loads parameters from fname into model, optimizer and config"""
    if fname.endswith('.gx'):
        load_gx(fname, model, optimizer, config)
    elif fname.endswith('.hgx'):
        load_hgx(fname, model, optimizer, config)
    else:
        raise IOError('Wrong file ending, should be .gx or .hgx')

    model.filename=os.path.abspath(fname)

def save_gx(fname, model, optimizer, config):
    model.save(fname)
    model.save_addition('config', config.model_dump())
    model.save_addition('optimizer',
                        optimizer.pickle_string(clear_evals=
                                                not config.get_boolean('solver',
                                                                       'save all evals')))

def save_hgx(fname, model, optimizer, config, group='current'):
    """ Saves the current objects to a hdf gx file (hgx).

    :param fname: filename
    :param model: model object
    :param optimizer: optimizer object
    :param config: config object
    :param group: name of the group, default current
    :return:
    """
    f=h5py.File(fname, 'w')
    g=f.create_group(group)
    model.write_h5group(g)
    try:
        clear_evals=not config.get_boolean('solver', 'save all evals')
    except OptionError as e:
        clear_evals=True
    optimizer.write_h5group(g.create_group('optimizer'), clear_evals=True)
    g['config']=config.model_dump().encode('utf-8')
    f.close()

def load_hgx(fname, model, optimizer, config, group='current'):
    """ Loads the current objects to a hdf gx file (hgx).

    :param fname: filename
    :param model: model object
    :param optimizer: optimizer object
    :param config: config object
    :param group: name of the group, default current
    :return:
    """
    f=h5py.File(fname, 'r')
    g=f[group]
    model.read_h5group(g)
    optimizer.read_h5group(g['optimizer'])
    config.load_model(g['config'][()].decode('utf-8'))
    f.close()

# Not yet used ...
def load_gx(fname, model, optimizer, config):
    if not 'diffev' in sys.modules:
        # for compatibility define genx standard modules as base modules
        import genx.diffev
        import genx.data
        import genx.model
        sys.modules['model']=genx.model
        sys.modules['diffev']=genx.diffev
        sys.modules['data']=genx.data
    model.load(fname)
    config.load_model(model.load_addition('config').decode('utf-8'))
    optimizer.pickle_load(model.load_addition('optimizer'))

# Functions to handle optimiser configs
# ==============================================================================


def load_opt_config(optimizer, config):
    """Load the config (Config class) values to the optimiser class (DiffEv class)."""

    class Container:
        error_bars_level=1.05
        save_all_evals=False

        def set_error_bars_level(self, val):
            self.error_bars_level=val

        def set_save_all_evals(self, val):
            self.save_all_evals=val

    c=Container()

    # Define all the options we want to set
    options_float=['km', 'kr', 'pop mult', 'pop size',
                   'max generations', 'max generation mult',
                   'sleep time', 'max log elements',
                   'errorbar level',
                   'autosave interval', 'parallel processes',
                   'parallel chunksize', 'allowed fom discrepancy']
    setfunctions_float=[optimizer.set_km, optimizer.set_kr,
                        optimizer.set_pop_mult,
                        optimizer.set_pop_size,
                        optimizer.set_max_generations,
                        optimizer.set_max_generation_mult,
                        optimizer.set_sleep_time,
                        optimizer.set_max_log,
                        c.set_error_bars_level,
                        optimizer.set_autosave_interval,
                        optimizer.set_processes,
                        optimizer.set_chunksize,
                        optimizer.set_fom_allowed_dis,
                        ]

    options_bool=['use pop mult', 'use max generations',
                  'use start guess', 'use boundaries',
                  'use parallel processing', 'use autosave',
                  'save all evals'
                  ]
    setfunctions_bool=[optimizer.set_use_pop_mult,
                       optimizer.set_use_max_generations,
                       optimizer.set_use_start_guess,
                       optimizer.set_use_boundaries,
                       optimizer.set_use_parallel_processing,
                       optimizer.set_use_autosave,
                       c.set_save_all_evals,
                       ]

    # Make sure that the config is set
    if config:
        # Start witht the float values
        for index in range(len(options_float)):
            try:
                val=config.get_float('solver', options_float[index])
            except OptionError as e:
                iprint('Could not locate option solver.'+options_float[index])
            else:
                setfunctions_float[index](val)

        # Then the bool flags
        for index in range(len(options_bool)):
            try:
                val=config.get_boolean('solver', options_bool[index])
            except OptionError as e:
                iprint('Could not read option solver.'+options_bool[index])
            else:
                setfunctions_bool[index](val)
        try:
            val=config.get('solver', 'create trial')
        except OptionError as e:
            iprint('Could not read option solver.create trial')
        else:
            try:
                optimizer.set_create_trial(val)
            except LookupError:
                iprint('The mutation scheme %s does not exist'%val)

    return c.error_bars_level, c.save_all_evals

def save_opt_config(optimizer, config, fom_error_bars_level=1.05, save_all_evals=False):
    """ Write the config values from optimizer (DiffEv class) to config (Config class) """

    # Define all the options we want to set
    options_float=['km', 'kr', 'pop mult', 'pop size',
                   'max generations', 'max generation mult',
                   'sleep time', 'max log elements', 'errorbar level',
                   'autosave interval',
                   'parallel processes', 'parallel chunksize',
                   'allowed fom discrepancy']
    set_float=[optimizer.km, optimizer.kr,
               optimizer.pop_mult,
               optimizer.pop_size,
               optimizer.max_generations,
               optimizer.max_generation_mult,
               optimizer.sleep_time,
               optimizer.max_log,
               fom_error_bars_level,
               optimizer.autosave_interval,
               optimizer.processes,
               optimizer.chunksize,
               optimizer.fom_allowed_dis
               ]

    options_bool=['use pop mult', 'use max generations',
                  'use start guess', 'use boundaries',
                  'use parallel processing', 'use autosave',
                  'save all evals',
                  ]
    set_bool=[optimizer.use_pop_mult,
              optimizer.use_max_generations,
              optimizer.use_start_guess,
              optimizer.use_boundaries,
              optimizer.use_parallel_processing,
              optimizer.use_autosave,
              save_all_evals,
              ]

    # Make sure that the config is set
    if config:
        # Start witht the float values
        for index in range(len(options_float)):
            try:
                config.set('solver', options_float[index], set_float[index])
            except OptionError as e:
                iprint('Could not locate save solver.'+options_float[index])

        # Then the bool flags
        for index in range(len(options_bool)):
            try:
                config.set('solver', options_bool[index], set_bool[index])
            except OptionError as e:
                iprint('Could not write option solver.'+options_bool[index])

        try:
            config.set('solver', 'create trial', optimizer.get_create_trial())
        except OptionError as e:
            iprint('Could not write option solver.create trial')

# ==============================================================================
class Config:
    def __init__(self):
        self.default_config=CP.ConfigParser()
        self.model_config=CP.ConfigParser()

    def load_default(self, filename, reset=False):
        '''load_default(self, filename) --> None
        
        Loads the default config from file filename. Raises a IOError if the
        can not be found.
        '''
        try:
            self.default_config.read(filename)
        except Exception as e:
            iprint(e)
            raise IOError('Could not load default config file', filename)
        if reset:
            self.model_config=CP.ConfigParser()

    def write_default(self, filename):
        '''write_default(self, filename) --> None
        
        Writes the current defualt configuration to filename
        '''
        try:
            with open(filename, 'w') as cfile:
                self.default_config.write(cfile)
        except Exception as e:
            iprint(e)
            raise IOError('Could not write default config file', filename)

    def load_model(self, str):
        '''load_model(self, str) --> None
        
        Loads a config from a string str.  Raises an IOError if the string can not be
        read.
        '''
        buffer=io.StringIO(str)
        self.model_config=CP.ConfigParser()
        try:
            self.model_config.read_file(buffer)
        except Exception as e:
            raise IOError('Could not load model config file')

    def _getf(self, default_function, model_function, section, option,
              fallback=None):
        '''_getf(default_function, model_function, section, option) --> object
        
        For the function function try to locate the section and option first in
        model_config if that fails try to locate it in default_config. If both
        fails raise an OptionError.
        '''
        value=0
        try:
            value=model_function(section, option)
        except Exception as e:
            try:
                value=default_function(section, option)
            except Exception as e:
                if fallback is None:
                    raise OptionError(section, option)
                else:
                    value=fallback
        return value

    def get_float(self, section, option):
        '''get_float(self, section, option) --> float
        
        returns a float value if possible for option in section
        '''
        return self._getf(self.default_config.getfloat,
                          self.model_config.getfloat, section, option)

    def get_boolean(self, section, option, fallback=None):
        '''get_boolean(self, section, option) --> boolean
        
        returns a boolean value if possible for option in section
        '''
        return self._getf(self.default_config.getboolean,
                          self.model_config.getboolean, section, option,
                          fallback=fallback)

    def get_int(self, section, option, fallback=None):
        '''get_int(self, section, option) --> int
        
        returns a int value if possible for option in section
        '''
        return self._getf(self.default_config.getint,
                          self.model_config.getint, section, option,
                          fallback=fallback)

    def get(self, section, option, fallback=None):
        '''get(self, section, option) --> string
        
        returns a string value if possible for option in section
        '''
        return self._getf(self.default_config.get,
                          self.model_config.get, section, option,
                          fallback=fallback)

    def model_set(self, section, option, value):
        '''model_set(self, section, option, value) --> None
        
        Set a value in section, option for the model configuration.
        '''
        if not self.model_config.has_section(section):
            self.model_config.add_section(section)
        self.model_config.set(section, option, str(value))
        # print 'Model set: ', section, ' ', option, ' ', value

    def default_set(self, section, option, value):
        '''model_set(self, section, option, value) --> None
        
        Set a value in section, option for the model configuration.
        '''
        if not self.default_config.has_section(section):
            self.default_config.add_section(section)
        self.default_config.set(section, option, str(value))
        # print 'Default set: ', section, ' ', option, ' ', value

    def set(self, section, option, value):
        '''set(self, section, option, value) --> None
        
        Set a value in section, option for the model configuration. 
        Just a duplication of model_set.
        '''
        self.model_set(section, option, value)

    def model_dump(self):
        '''model_save(self, file_pointer) --> string
        
        dumps the model configuration to a string.
        '''
        # Create a buffer - file like object to trick config parser
        buffer=io.StringIO()
        # write
        self.model_config.write(buffer)
        # get the string values
        str=buffer.getvalue()
        # Close the buffer - destroy it
        buffer.close()
        # print 'model content: ', str
        return str

# END: class Config
# ==============================================================================
# Some Exception definition for errorpassing
class GenericError(Exception):
    ''' Just a empty class used for inheritance. Only useful
    to check if the errors are originating from the model library.
    All these errors are controllable. If they not originate from
    this class something has passed trough and that should be impossible '''
    pass

class IOError(GenericError):
    ''' Error class for input output, mostly concerning files'''

    def __init__(self, error_message, file=''):
        '''__init__(self, error_message)'''
        self.error_message=error_message
        self.file=file

    def __str__(self):
        text='Input/Output error for file:\n'+self.file+ \
             '\n\n Python error:\n '+self.error_message

class OptionError(GenericError):
    ''' Error class for not finding an option section pair in the 
    configuration '''

    def __init__(self, section, option):
        '''__init__(self, error_message)'''
        # self.error_message = error_message
        self.section=section
        self.option=option

    def __str__(self):
        text='Error in trying to loacate values in GenX configuration.'+ \
             '\nCould not locate the section: '+self.section+ \
             ' or option: '+self.option+'.'
        return text



'''
==========================================
:mod:`orso` ORSO standard file data loader
==========================================

Loads the format following the specification of the Open Reflectometry Standards Organization (ORSO).
See https://www.reflectometry.org/working_groups/file_formats/ for more details
'''

import numpy as np
import os

from ..data_loader_framework import Template
from ..utils import ShowWarningDialog
from genx.lib.orso_io.ort import read_file

class Plugin(Template):
    wildcard='*.ort'

    def __init__(self, parent):
        Template.__init__(self, parent)
        self.x_col=0
        self.y_col=1
        self.e_col=3
        self.xe_col=2
        self.comment='#'
        self.skip_rows=0
        self.delimiter=None

    def CanOpen(self, file_path):
        if not Template.CanOpen(self, file_path):
            return False
        l1=open(file_path, 'r', encoding='utf-8').readline()
        return l1.startswith('# ORSO')

    def LoadData(self, dataset, filename, data_id=0):
        '''LoadData(self, data_item_number, filename) --> none

        Loads the data from filename into the data_item_number.
        '''
        try:
            ord_data=read_file(filename)
        except Exception as e:
            ShowWarningDialog(self.parent, 'Could not load the file: '+filename+
                              ' \nPlease check the format.\n\n ORSO reader gave the following error:\n'+str(e))
            return
        else:
            if type(ord_data) is list:
                ord_data=ord_data[data_id]

            dataset.x_raw=np.asarray(ord_data.x)
            dataset.y_raw=np.asarray(ord_data.y)
            dataset.error_raw=np.asarray(ord_data.dy)
            if ord_data.dx is not None:
                dataset.set_extra_data('res', np.asarray(ord_data.dx), 'res')
            for col in ord_data[4:]:
                dataset.set_extra_data(col.name, np.asarray(col), col.name)
            # Name the dataset accordign to file name
            if ord_data.name.strip()!='':
                dataset.name=ord_data.name
            # Run the commands on the data - this also sets the x,y, error memebers
            # of that data item.
            dataset.run_command()

            # insert metadata into ORSO compatible fields
            dataset.meta=dict(ord_data.header)
            del(dataset.meta['columns'])
            if 'data_set' in dataset.meta:
                del(dataset.meta['data_set'])



from numpy import *
import numba
import math, cmath

@numba.jit(numba.float64[:](numba.float64[:], numba.float64, numba.complex128[:], numba.float64[:], numba.float64[:]),
           nopython=True, parallel=True, cache=True)
def ReflNB(theta, lamda, n, d, sigma):
    layers=d.shape[0]
    angles=theta.shape[0]
    n0=n[-1]

    k=2.*pi/lamda
    R=empty_like(theta)

    pre1=2.*n0*k
    pre2=(pi/180.)

    for ai in numba.prange(angles):
        Qi=pre1*cmath.sqrt((n[0]/n0)**2-math.cos(theta[ai]*pre2)**2)

        Qj=pre1*cmath.sqrt((n[1]/n0)**2-math.cos(theta[ai]*pre2)**2)
        # Fresnel reflectivity for the interfaces
        rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[0]**2)
        Aj=rpj
        Qi=Qj

        for lj in range(2, layers):
            Qj=pre1*cmath.sqrt((n[lj]/n0)**2-math.cos(theta[ai]*pre2)**2)
            # Fresnel reflectivity for the interfaces
            rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[lj-1]**2)

            pj=cmath.exp(1.0j*d[lj-1]*Qi)
            part=Aj*pj
            Aj=(rpj+part)/(1.+part*rpj)

            Qi=Qj
        R[ai]=abs(Aj)**2
    return R

@numba.jit(
    numba.complex128[:](numba.float64[:], numba.float64, numba.complex128[:], numba.float64[:], numba.float64[:]),
    nopython=True, parallel=True, cache=True)
def AmpNB(theta, lamda, n, d, sigma):
    layers=d.shape[0]
    angles=theta.shape[0]
    n0=n[-1]

    k=2.*pi/lamda
    A=empty(theta.shape, dtype=complex128)

    pre1=2.*n0*k
    pre2=(pi/180.)

    for ai in numba.prange(angles):
        Qi=pre1*cmath.sqrt((n[0]/n0)**2-math.cos(theta[ai]*pre2)**2)

        Qj=pre1*cmath.sqrt((n[1]/n0)**2-math.cos(theta[ai]*pre2)**2)
        # Fresnel reflectivity for the interfaces
        rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[0]**2)
        Aj=rpj
        Qi=Qj

        for lj in range(2, layers):
            Qj=pre1*cmath.sqrt((n[lj]/n0)**2-math.cos(theta[ai]*pre2)**2)
            # Fresnel reflectivity for the interfaces
            rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[lj-1]**2)

            pj=cmath.exp(1.0j*d[lj-1]*Qi)
            part=Aj*pj
            Aj=(rpj+part)/(1.+part*rpj)

            Qi=Qj
        A[ai]=Aj
    return A

def Refl(theta, lamda, n, d, sigma, return_int=True):
    if return_int:
        return ReflNB(theta, lamda, n, d, sigma)
    else:
        return AmpNB(theta, lamda, n, d, sigma)

@numba.jit(numba.float64[:](numba.float64[:], numba.float64, numba.complex128[:], numba.float64[:], numba.float64[:]),
           nopython=True, parallel=True, cache=True)
def ReflQNB(Q, lamda, n, d, sigma):
    layers=d.shape[0]
    points=Q.shape[0]
    n0=n[-1]

    Q0=4.*pi/lamda
    R=empty_like(Q)

    for qi in numba.prange(points):
        Qi=cmath.sqrt((n[0]**2-n0**2)*Q0**2+n0**2*Q[qi]**2)

        Qj=cmath.sqrt((n[1]**2-n0**2)*Q0**2+n0**2*Q[qi]**2)
        # Fresnel reflectivity for the interfaces
        rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[0]**2)
        Aj=rpj
        Qi=Qj

        for lj in range(2, layers):
            Qj=cmath.sqrt((n[lj]**2-n0**2)*Q0**2+n0**2*Q[qi]**2)
            # Fresnel reflectivity for the interfaces
            rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[lj-1]**2)

            pj=cmath.exp(1.0j*d[lj-1]*Qi)
            part=Aj*pj
            Aj=(rpj+part)/(1.+part*rpj)

            Qi=Qj
        R[qi]=abs(Aj)**2
    return R

@numba.jit(
    numba.complex128[:](numba.float64[:], numba.float64, numba.complex128[:], numba.float64[:], numba.float64[:]),
    nopython=True, parallel=True, cache=True)
def AmpQNB(Q, lamda, n, d, sigma):
    layers=d.shape[0]
    points=Q.shape[0]
    n0=n[-1]

    Q0=4.*pi/lamda
    A=empty(Q.shape, dtype=complex128)

    for qi in numba.prange(points):
        Qi=cmath.sqrt((n[0]**2-n0**2)*Q0**2+n0**2*Q[qi]**2)

        Qj=cmath.sqrt((n[1]**2-n0**2)*Q0**2+n0**2*Q[qi]**2)
        # Fresnel reflectivity for the interfaces
        rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[0]**2)
        Aj=rpj
        Qi=Qj

        for lj in range(2, layers):
            Qj=cmath.sqrt((n[lj]**2-n0**2)*Q0**2+n0**2*Q[qi]**2)
            # Fresnel reflectivity for the interfaces
            rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[lj-1]**2)

            pj=cmath.exp(1.0j*d[lj-1]*Qi)
            part=Aj*pj
            Aj=(rpj+part)/(1.+part*rpj)

            Qi=Qj
        A[qi]=Aj
    return A

def ReflQ(Q, lamda, n, d, sigma, return_int=True):
    if return_int:
        return ReflQNB(Q, lamda, n, d, sigma)
    else:
        return AmpQNB(Q, lamda, n, d, sigma)

@numba.jit(
    numba.float64[:](numba.float64[:], numba.float64[:], numba.complex128[:, :], numba.float64[:], numba.float64[:]),
    nopython=True, parallel=True, cache=True)
def Refl_nvary2NB(theta, lamda, n, d, sigma):
    layers=d.shape[0]
    angles=theta.shape[0]

    R=empty_like(theta)

    pre2=(pi/180.)

    for ai in numba.prange(angles):
        n0=n[-1, ai]
        ki=2.*pi/lamda[ai]
        pre1=2.*n0*ki

        Qi=pre1*cmath.sqrt((n[0, ai]/n0)**2-math.cos(theta[ai]*pre2)**2)

        Qj=pre1*cmath.sqrt((n[1, ai]/n0)**2-math.cos(theta[ai]*pre2)**2)
        # Fresnel reflectivity for the interfaces
        rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[0]**2)
        Aj=rpj
        Qi=Qj

        for lj in range(2, layers):
            Qj=pre1*cmath.sqrt((n[lj, ai]/n0)**2-math.cos(theta[ai]*pre2)**2)
            # Fresnel reflectivity for the interfaces
            rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[lj-1]**2)

            pj=cmath.exp(1.0j*d[lj-1]*Qi)
            part=Aj*pj
            Aj=(rpj+part)/(1.+part*rpj)

            Qi=Qj
        R[ai]=abs(Aj)**2
    return R

@numba.jit(
    numba.complex128[:](numba.float64[:], numba.float64[:], numba.complex128[:, :], numba.float64[:], numba.float64[:]),
    nopython=True, parallel=True, cache=True)
def Amp_nvary2NB(theta, lamda, n, d, sigma):
    layers=d.shape[0]
    angles=theta.shape[0]

    A=empty(theta.shape, dtype=complex128)

    pre2=(pi/180.)

    for ai in numba.prange(angles):
        n0=n[-1, ai]
        ki=2.*pi/lamda[ai]
        pre1=2.*n0*ki

        Qi=pre1*cmath.sqrt((n[0, ai]/n0)**2-math.cos(theta[ai]*pre2)**2)

        Qj=pre1*cmath.sqrt((n[1, ai]/n0)**2-math.cos(theta[ai]*pre2)**2)
        # Fresnel reflectivity for the interfaces
        rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[0]**2)
        Aj=rpj
        Qi=Qj

        for lj in range(2, layers):
            Qj=pre1*cmath.sqrt((n[lj, ai]/n0)**2-math.cos(theta[ai]*pre2)**2)
            # Fresnel reflectivity for the interfaces
            rpj=(Qj-Qi)/(Qj+Qi)*cmath.exp(-Qj*Qi/2.*sigma[lj-1]**2)

            pj=cmath.exp(1.0j*d[lj-1]*Qi)
            part=Aj*pj
            Aj=(rpj+part)/(1.+part*rpj)

            Qi=Qj
        A[ai]=Aj
    return A

def Refl_nvary2(theta, lamda, n, d, sigma, return_int=True):
    if return_int:
        return Refl_nvary2NB(theta, lamda, n, d, sigma)
    else:
        return Amp_nvary2NB(theta, lamda, n, d, sigma)



import argparse
import re
import os
import time
import sys

import numpy as np

from . import version
from .gui_logging import iprint

def sld_mc(args):
    """ Function to start fitting from the command line.

    :param args:
    :return:
    """
    from . import model
    from . import diffev
    from . import filehandling as io

    mod=model.Model()
    config=io.Config()
    config.load_default(os.path.split(os.path.abspath(__file__))[0]+'genx.conf')
    opt=diffev.DiffEv()

    iprint('Loading model %s...'%args.infile)
    io.load_file(args.infile, mod, opt, config)
    io.load_opt_config(opt, config)
    # Hack the script so that it simulates the SLD instead of the data
    mod.script=re.sub(r"SimSpecular\((.*)\,(.*)\)", r"SimSLD(data[0].x, None,\2)", mod.script)
    iprint("Hacking the model script. Resulting script:")
    iprint(mod.script)

    # Simulate, this will also compile the model script
    iprint('Compiling model...')
    mod.compile_script()

    (funcs, vals, minvals, maxvals)=mod.get_fit_pars()
    vals=np.array(vals)
    boundaries=[row[5] for row in mod.parameters.data if not row[0]=='' and row[2]]
    iprint(boundaries)
    boundaries=np.array([eval(s) for s in boundaries])
    minvals, maxvals=boundaries[:, 0]+vals, boundaries[:, 1]+vals
    min_SLD=[]
    max_SLD=[]
    z=np.arange(args.min, args.max, args.step)
    # Change the x-data so that it contain the z values instead.
    for d in mod.data:
        d.x=z

    def extreme_dict(cur, extreme, func):
        """Makes a comparison of cur and extreme through func (np.min or np.max) and returns the result as a dict"""
        return_dic={}
        for key in extreme:
            if key not in ['z', 'SLD unit']:
                # print cur[key].shape
                return_dic[key]=func(np.c_[cur[key], extreme[key]], axis=1)
            else:
                return_dic[key]=cur[key]
        return return_dic

    iprint("Calculating sld's...")
    missed=0
    for i in range(args.runs):
        current_vals=minvals+(maxvals-minvals)*np.random.random_sample(len(funcs))
        [func(val) for func, val in zip(funcs, current_vals)]

        mod.script_module._sim=False
        current_sld=mod.script_module.Sim(mod.data)
        same_shape=all([sld['z'].shape==msld['z'].shape for sld, msld in zip(current_sld, min_SLD)])
        if i==0:
            min_SLD=[sld for sld in current_sld]
            max_SLD=[sld for sld in current_sld]
        elif same_shape:
            min_SLD=[extreme_dict(sld, msld, np.min) for sld, msld in zip(current_sld, min_SLD)]
            max_SLD=[extreme_dict(sld, msld, np.max) for sld, msld in zip(current_sld, max_SLD)]
        else:
            missed+=1

        sys.stdout.write("\r Progress: %d%%"%(float(i)*100/float(args.runs)))
        sys.stdout.flush()

    iprint(' ')
    iprint(missed, " simulations was discarded due to wrong size.")
    iprint("Saving the data to file...")
    for sim in range(len(min_SLD)):
        new_filename=(args.outfile+'%03d'%sim+'.dat')
        save_array=np.array([min_SLD[sim]['z']])
        header='z\t'
        for key in min_SLD[sim]:
            if key!='z' and key!='SLD unit':
                save_array=np.r_[save_array, [min_SLD[sim][key].real], [min_SLD[sim][key].imag],
                                 [max_SLD[sim][key].real], [max_SLD[sim][key].imag]]
                header+='min(%s.real)\tmin(%s.imag)\tmax(%s.real)\tmax(%s.imag)\t'%(key, key, key, key)
        with open(new_filename, 'w') as f:
            f.write(
                "# Monte Carlo estimation of SLD bounds with script sld_errorbars.py model taken from file: %s\n"%args.infile)
            f.write("# File created: %s\n"%time.ctime())
            f.write("# Simulated SLD for data set: %s\n"%mod.data[sim].name)
            f.write("# Headers: \n")
            f.write('#'+header+'\n')
            np.savetxt(f, save_array.transpose(), delimiter='\t')

if __name__=='__main__':
    parser=argparse.ArgumentParser(description="sld_errorbars %s, creates a boundary sld profile from given errorbar"
                                               " values in a GenX file. Note this is experimental software!"%version.__version__,
                                   epilog="For support, manuals and bug reporting see http://genx.sf.net"
                                   )

    parser.add_argument('--runs', type=int, default=1, help='Number of Monte Carlo evaluations')
    parser.add_argument('--min', type=float, default=0.0, help='Minimum position to evaluate SLD')
    parser.add_argument('--max', type=float, default=1000.0, help='Maximum position to evaluate SLD')
    parser.add_argument('--step', type=float, default=0.5, help='Step size for the gridding of the SLD')

    parser.add_argument('infile', default='', help='The .gx or .hgx file to load')
    parser.add_argument('outfile', default='out', help='The output base file name (no extension)')

    args=parser.parse_args()
    sld_mc(args)



# Functions for instrumental resolution corrections and area of illumination
# these are to be included in the Models class and Refl class
# Note that these function assumes a Gaussian spread in both the angular as well
# as in real space

# THis one really needs scipy
from numpy import *
from scipy.special import erf
from genx.gui_logging import iprint

rad=pi/180.
sqrt2=sqrt(2.)

# Intensity correction Gaussian beamprofile
def GaussIntensity(alpha, s1, s2, sigma_x):
    sinalpha=sin(alpha*rad)
    if s1==s2:
        return erf(s2/sqrt2/sigma_x*sinalpha)
    else:
        common=sinalpha/sqrt2/sigma_x
        return (erf(s2*common)+erf(s1*common))/2.0

# Specular foorprintcorrections square beamprofile
def SquareIntensity(alpha, slen, beamwidth):
    F=slen/beamwidth*sin(alpha*rad)
    return where(F<=1.0, F, ones(F.shape))

####################################################################
## Resolution Functions (Normal Distributed=Gaussian)
#####################################################################

# Full Convlutions - vayring resolution

# Function to create a 1D vector for the resolution with the 
# positions to calculate the reflectivity Qret and the weight
# of each point weight
# Inputs: Q - the Q values 
#         dQ - the resolution
#         points - the number of points for the convolution
#         range how far the gaussian should be convoluted
def ResolutionVector(Q, dQ, points, range=3):
    # if type(dQ)!=type(array([])):
    #    dQ=dQ*ones(Q.shape)
    Qres=Q+dQ*linspace(-range, range, points)[:, newaxis]

    weight=1/sqrt(2*pi)/dQ*exp(-(transpose(Q[:, newaxis])-Qres)**2/dQ**2/2)
    Qret=Qres.flatten()  # reshape(Qres,(1,Qres.shape[0]*Qres.shape[1]))[0]
    # print Qres
    # print Qres.shape
    # print Qret.shape
    return Qret, weight

# Include the resolution with Qret and weight calculated from ResolutionVector
# and I the calculated intensity at each point. returns the intensity
def ConvoluteResolutionVector(Qret, I, weight):
    Qret2=Qret.reshape(weight.shape[0], weight.shape[1])
    # print Qret.shape,weight.shape
    I2=I.reshape(weight.shape[0], weight.shape[1])
    # print (I*weight).shape,Qret.shape
    norm_fact=trapz(weight, x=Qret2, axis=0)
    Int=trapz(I2*weight, x=Qret2, axis=0)/norm_fact
    # print Int.shape
    return Int

# Fast convlution - constant resolution
# constant spacing between data!
def ConvoluteFast(Q, I, dQ, range=3):
    Qstep=Q[1]-Q[0]
    resvector=arange(-range*dQ, range*dQ+Qstep, Qstep)
    weight=1/sqrt(2*pi)/dQ*exp(-resvector**2/dQ**2/2)
    Iconv=convolve(r_[ones(resvector.shape)*I[0], I, ones(resvector.shape)*I[-1]], weight/weight.sum(),
                   mode='same')[resvector.shape[0]:-resvector.shape[0]]
    return Iconv

# Fast convolution - varying resolution
# constant spacing between the dat.
def ConvoluteFastVar(Q, I, dQ, range=3):
    Qstep=Q[1]-Q[0]
    steps=max(dQ*ones(Q.shape))*range/Qstep
    weight=1/sqrt(2*pi)/dQ*exp(-(Q[:, newaxis]-Q)**2/dQ**2/2)
    Itemp=I[:, newaxis]*ones(I.shape)
    norm_fact=trapz(weight, axis=0)
    Int=trapz(Itemp*weight, axis=0)/norm_fact
    return Int

def QtoTheta(wavelength, Q):
    return arcsin(wavelength/4.0/pi*Q)/rad

def TwoThetatoQ(wavelength, TwoTheta):
    return 4.0*pi/wavelength*sin(rad/2.*TwoTheta)

from . import USE_NUMBA

if USE_NUMBA:
    # try to use numba to speed up the calculation intensive functions:
    try:
        from .instrument_numba import GaussIntensity, SquareIntensity, QtoTheta, TwoThetatoQ, ResolutionVector
    except Exception as e:
        iprint('Could not use numba, no speed up from JIT compiler:\n'+str(e))



'''<h1>Exporter</h1>
A plugin that allows to export models for the use in other programs.
Currently, only BornAgain is supported to easy transfer from spcular
to off-specular or GISAS simulations.

'''
from .. import add_on_framework as framework
import wx

class Plugin(framework.Template):
    def __init__(self, parent):
        framework.Template.__init__(self, parent)
        self.menu=self.NewMenu('Exporter')
        self.parent=parent

        self.mb_export_ba=wx.MenuItem(self.menu, wx.NewId(),
                                      "BornAgain script...",
                                      "Export reflectometry layers to BornAgain python script.",
                                      wx.ITEM_NORMAL)
        self.menu.Append(self.mb_export_ba)
        self.parent.Bind(wx.EVT_MENU, self.OnExportBA, self.mb_export_ba)

        self.StatusMessage('Sucessfully loaded Exporter...')

    def OnExportBA(self, event):
        '''Export layer model to BornAgain script.
        '''
        dlg=wx.FileDialog(self.parent, message="Export As", defaultFile="genx_model.py",
                          wildcard="Python (*.py)|*.py",
                          style=wx.FD_SAVE  # | wx.FD_CHANGE_DIR
                          )
        if dlg.ShowModal()!=wx.ID_OK:
            return
        fname=dlg.GetPath()

        model=self.GetModel()
        model.simulate()
        m=model.script_module
        names=list(m.__dict__.keys())
        objects=list(m.__dict__.values())

        s=m.sample

        output='# layer definitions: thickness, density, f-xray, b-neutron, roughness, muB/FU\n'
        output+='Ambient='+self._expand_layer(s.Ambient)+'\n'
        output+='Stacks=[\n'
        for stack in s.Stacks:
            if stack in objects:
                name='- Stack: '+names[objects.index(stack)]
            else:
                name=''
            output+=f"         [{stack.Repetitions:d}, #repetitions {name}\n"
            for l in stack.Layers:
                if l in objects:
                    name='# '+names[objects.index(l)]
                else:
                    name=''
                output+='           '+self._expand_layer(l)+f',{name}\n'
            output+='         ],\n'
        output+='       ]\n'
        output+='Substrate='+self._expand_layer(s.Substrate)+'\n\n'
        output+=f'probe="{m.inst.probe}"'

        open(fname, 'w', encoding='utf-8').write(TEMPLATE%output)

    def _expand_layer(self, layer):
        return f"[{layer.d}, {layer.dens}, {layer.f}, {layer.b}, {layer.sigma}, {layer.magn}]"

    def clear_menu(self):
        '''clear_menu(self) --> None
        
        Clears the menu from all items present in it
        '''
        [self.menu.RemoveItem(item) for item in self.menu.GetMenuItems()]

TEMPLATE='''import bornagain as ba
from bornagain import angstrom, deg, nm

%s

muB=9274.0101e8 #BohrMagneton/1e-5nm³ to A∕m

def get_sample():
    """
    Builds the sample from the defined GenX model.
    """
    hurst=0.4
    corr=100.*nm
    magnetizationVector    = ba.kvector_t(0, muB,0)
    multi_layer = ba.MultiLayer()
    
    if probe=="x-ray":
        midx=2
    else:
        midx=3

    m_ambient = ba.MaterialBySLD("Ambient", 1e-5*Ambient[1]*Ambient[midx].real, -1e-5*Ambient[1]*Ambient[midx].imag)
    multi_layer.addLayer(ba.Layer(m_ambient))
    
    for i, stack in enumerate(Stacks):
        j=0
        for rep in range(stack[0]):
            for li in stack[1:]:
                if 'pol' in probe and li[5]>0:
                    material=ba.MaterialBySLD("S%%iL%%i"%%(i,j), 1e-5*li[1]*li[midx].real, -1e-5*li[1]*li[midx].imag, 1e-5*magnetizationVector*li[1]*li[5])
                else:
                    material=ba.MaterialBySLD("S%%iL%%i"%%(i,j), 1e-5*li[1]*li[midx].real, -1e-5*li[1]*li[midx].imag)
                if li[4]>0:
                    roughness=ba.LayerRoughness(li[4]*angstrom, hurst, corr)
                    multi_layer.addLayerWithTopRoughness(ba.Layer(material, li[0]*angstrom), roughness)
                else:
                    multi_layer.addLayer(ba.Layer(material, li[0]*angstrom))
                j+=1
    
    
    m_substrate = ba.MaterialBySLD("Substrate", 1e-5*Substrate[1]*Substrate[midx].real, -1e-5*Substrate[1]*Substrate[midx].imag)
    if Substrate[4]>0:
        roughness=ba.LayerRoughness(Substrate[4]*angstrom, hurst, corr)
        multi_layer.addLayerWithTopRoughness(ba.Layer(m_substrate), roughness)
    else:
        multi_layer.addLayer(ba.Layer(m_substrate))
    
    return multi_layer

def get_simulation(scan_size=500):
    """
    Defines and returns specular simulation
    with a qz-defined beam
    """
    qzs = np.linspace(0.01, 3.0, scan_size)  # qz-values
    dq = 0.03 * qzs
    n_sig = 2.0
    n_samples = 25

    distr = ba.RangedDistributionGaussian(n_samples, n_sig)

    scan = ba.QSpecScan(qzs)
    scan.setAbsoluteQResolution(distr, dq)

    simulation = ba.SpecularSimulation()
    simulation.setScan(scan)

    return simulation

def run_simulation(polarization=ba.kvector_t(0.0, 1.0, 0.0),
                   analyzer=None):
    """
    Runs simulation and returns its result.
    """
    sample = get_sample()
    simulation = get_simulation()
    
    # adding polarization and analyzer operator
    if polarization:
        simulation.setBeamPolarization(polarization)
    if analyzer:
      simulation.setAnalyzerProperties(analyzer, 1.0, 0.5)
    
    simulation.setSample(sample)
    simulation.runSimulation()
    return simulation.result()

if __name__=='__main__':
    from numpy import array
    from matplotlib.pyplot import *
    smpl=get_sample()
    figure(figsize=(16,8))
    subplot(121)
    title('Reflectivity %%s'%%probe)
    if 'pol' in probe:
        ba.plot_simulation_result(run_simulation(polarization=ba.kvector_t(0.0, 1.0, 0.0)), 
            label='up', postpone_show=True)
        ba.plot_simulation_result(run_simulation(polarization=ba.kvector_t(0.0, -1.0, 0.0)), 
            label='down', postpone_show=True)
        legend()
    else:
        ba.plot_simulation_result(run_simulation(polarization=None), postpone_show=True)
    subplot(122)
    title('Scattering Length Density Profile')
    z, sld=ba.MaterialProfile(smpl)
    plot(z,array(sld).real)
    xlabel("z [nm]")
    ylabel("SLD [$\\AA^{-2}$]")
    tight_layout()
    show()
'''
